@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The command line interface for the node API
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -22,10 +22,22 @@
 #define NUMBUFSIZ      32                /* size of a buffer for a number */
 #define IOBUFSIZ       8192              /* size of a buffer for I/O */
 #define MINIBNUM       31                /* bucket number of a small map */
+#define RESLIMSIZE     (1048576*128)     /* limitation of the entity body of response */
 #define SEARCHMAX      10                /* maximum number of shown documents */
+#define SEARCHAUX      32                /* minimum hits to adopt the auxiliary index */
+#define DATTRNDURL     "#nodeurl"        /* name of the pseudo-attribute of the node URL */
+#define DATTRNDSCORE   "#nodescore"      /* name of the pseudo-attribute of the node score */
+
+enum {                                   /* enumeration for informing modes */
+  IM_BASIC,                              /* basical information */
+  IM_ADMINS,                             /* list of administrators */
+  IM_USERS,                              /* list of users */
+  IM_LINKS                               /* list of links */
+};
 
 enum {                                   /* enumeration for viewing modes */
   VM_PROT,                               /* protocol simulation */
+  VM_URI,                                /* ID and URI */
   VM_XML                                 /* XML */
 };
 
@@ -37,6 +49,7 @@ int g_pxport = 0;                        /* host name of the proxy */
 int g_timeout = -1;                      /* timeout in seconds */
 const char *g_authname = NULL;           /* name of authority */
 const char *g_authpass = NULL;           /* password of authority */
+int g_infomode = IM_BASIC;               /* informing mode */
 int g_viewmode = VM_PROT;                /* viewing mode */
 int g_nonparse = FALSE;                  /* whether to output response headers */
 CBLIST *g_exheaders = NULL;              /* extension headers */
@@ -53,6 +66,8 @@ static int runget(int argc, char **argv);
 static int runetch(int argc, char **argv);
 static int runuriid(int argc, char **argv);
 static int runinform(int argc, char **argv);
+static int runsync(int argc, char **argv);
+static int runoptimize(int argc, char **argv);
 static int runsearch(int argc, char **argv);
 static int runlist(int argc, char **argv);
 static int runsetuser(int argc, char **argv);
@@ -67,8 +82,10 @@ static int procget(const char *nurl, int id, const char *uri, const char *attr);
 static int procetch(const char *nurl, int id, const char *uri);
 static int procuriid(const char *nurl, const char *uri);
 static int procinform(const char *nurl);
+static int procsync(const char *nurl);
+static int procoptimize(const char *nurl);
 static int procsearch(const char *nurl, const char *phrase, const CBLIST *attrs, const char *ord,
-                      int max, int opts, int depth, double ec);
+                      int max, int sk, int aux, int opts, int depth, int mask, double ec);
 static int proclist(const char *nurl);
 static int procsetuser(const char *nurl, const char *name, int mode);
 static int procsetlink(const char *nurl, const char *url, const char *label, int credit);
@@ -78,7 +95,7 @@ static int procraw(const char *url, const char *file);
 /* main routine */
 int main(int argc, char **argv){
   int rv;
-  cbstdiobin();
+  est_proc_env_reset();
   g_progname = argv[0];
   if(!est_init_net_env()){
     printferror("could not initialize network environment");
@@ -101,6 +118,10 @@ int main(int argc, char **argv){
     rv = runuriid(argc, argv);
   } else if(!strcmp(argv[1], "inform")){
     rv = runinform(argc, argv);
+  } else if(!strcmp(argv[1], "sync")){
+    rv = runsync(argc, argv);
+  } else if(!strcmp(argv[1], "optimize")){
+    rv = runoptimize(argc, argv);
   } else if(!strcmp(argv[1], "search")){
     rv = runsearch(argc, argv);
   } else if(!strcmp(argv[1], "list")){
@@ -136,10 +157,14 @@ static void usage(void){
   fprintf(stderr, "  %s uriid [-proxy host port] [-tout num] [-auth user pass]"
           " nurl uri\n", g_progname);
   fprintf(stderr, "  %s inform [-proxy host port] [-tout num] [-auth user pass]"
+          " [-ia|-iu|-il] nurl\n", g_progname);
+  fprintf(stderr, "  %s sync [-proxy host port] [-tout num] [-auth user pass]"
+          " nurl\n", g_progname);
+  fprintf(stderr, "  %s optimize [-proxy host port] [-tout num] [-auth user pass]"
           " nurl\n", g_progname);
   fprintf(stderr, "  %s search [-proxy host port] [-tout num] [-auth user pass]"
-          " [-vx] [-kw] [-ec rn] [-sf] [-attr expr] [-ord expr] [-max num] [-dpt num]"
-          " nurl [phrase]\n", g_progname);
+          " [-vu|-vx] [-ec rn] [-sf] [-attr expr] [-ord expr] [-max num] [-sk num] [-aux num]"
+          " [-dpt num] [-mask num] nurl [phrase]\n", g_progname);
   fprintf(stderr, "  %s list [-proxy host port] [-tout num] [-auth user pass]"
           " nurl\n", g_progname);
   fprintf(stderr, "  %s setuser [-proxy host port] [-tout num] [-auth user pass]"
@@ -428,6 +453,12 @@ static int runinform(int argc, char **argv){
         g_authname = argv[i];
         if(++i >= argc) usage();
         g_authpass = argv[i];
+      } else if(!strcmp(argv[i], "-ia")){
+        g_infomode = IM_ADMINS;
+      } else if(!strcmp(argv[i], "-iu")){
+        g_infomode = IM_USERS;
+      } else if(!strcmp(argv[i], "-il")){
+        g_infomode = IM_LINKS;
       } else {
         usage();
       }
@@ -443,18 +474,91 @@ static int runinform(int argc, char **argv){
 }
 
 
+/* parse arguments of the sync command */
+static int runsync(int argc, char **argv){
+  char *nurl;
+  int i, rv;
+  nurl = NULL;
+  for(i = 2; i < argc; i++){
+    if(!nurl && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-proxy")){
+        if(++i >= argc) usage();
+        g_pxhost = argv[i];
+        if(++i >= argc) usage();
+        g_pxport = atoi(argv[i]);
+      } else if(!strcmp(argv[i], "-tout")){
+        if(++i >= argc) usage();
+        g_timeout = atoi(argv[i]);
+      } else if(!strcmp(argv[i], "-auth")){
+        if(++i >= argc) usage();
+        g_authname = argv[i];
+        if(++i >= argc) usage();
+        g_authpass = argv[i];
+      } else {
+        usage();
+      }
+    } else if(!nurl){
+      nurl = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!nurl) usage();
+  rv = procsync(nurl);
+  return rv;
+}
+
+
+/* parse arguments of the optimize command */
+static int runoptimize(int argc, char **argv){
+  char *nurl;
+  int i, rv;
+  nurl = NULL;
+  for(i = 2; i < argc; i++){
+    if(!nurl && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-proxy")){
+        if(++i >= argc) usage();
+        g_pxhost = argv[i];
+        if(++i >= argc) usage();
+        g_pxport = atoi(argv[i]);
+      } else if(!strcmp(argv[i], "-tout")){
+        if(++i >= argc) usage();
+        g_timeout = atoi(argv[i]);
+      } else if(!strcmp(argv[i], "-auth")){
+        if(++i >= argc) usage();
+        g_authname = argv[i];
+        if(++i >= argc) usage();
+        g_authpass = argv[i];
+      } else {
+        usage();
+      }
+    } else if(!nurl){
+      nurl = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!nurl) usage();
+  rv = procoptimize(nurl);
+  return rv;
+}
+
+
 /* parse arguments of the search command */
 static int runsearch(int argc, char **argv){
   CBDATUM *pbuf;
   CBLIST *attrs;
   char *nurl, *ord;
-  int i, max, opts, depth, rv;
+  int i, max, sk, aux, opts, depth, mask, rv;
   double ec;
   nurl = NULL;
   ord = NULL;
   max = SEARCHMAX;
+  sk = 0;
+  aux = SEARCHAUX;
   opts = 0;
   depth = 0;
+  mask = 0;
   ec = -1.0;
   pbuf = cbdatumopen("", 0);
   cbglobalgc(pbuf, (void (*)(void *))cbdatumclose);
@@ -475,14 +579,13 @@ static int runsearch(int argc, char **argv){
         g_authname = argv[i];
         if(++i >= argc) usage();
         g_authpass = argv[i];
+      } else if(!strcmp(argv[i], "-vu")){
+        g_viewmode = VM_URI;
       } else if(!strcmp(argv[i], "-vx")){
         g_viewmode = VM_XML;
-      } else if(!strcmp(argv[i], "-kw")){
-        opts |= ESTCONDETCH;
       } else if(!strcmp(argv[i], "-ec")){
         if(++i >= argc) usage();
         ec = strtod(argv[i], NULL);
-        opts |= ESTCONDETCH;
       } else if(!strcmp(argv[i], "-sf")){
         opts |= ESTCONDSIMPLE;
       } else if(!strcmp(argv[i], "-attr")){
@@ -494,9 +597,18 @@ static int runsearch(int argc, char **argv){
       } else if(!strcmp(argv[i], "-max")){
         if(++i >= argc) usage();
         max = atoi(argv[i]);
+      } else if(!strcmp(argv[i], "-sk")){
+        if(++i >= argc) usage();
+        sk = atoi(argv[i]);
+      } else if(!strcmp(argv[i], "-aux")){
+        if(++i >= argc) usage();
+        aux = atoi(argv[i]);
       } else if(!strcmp(argv[i], "-dpt")){
         if(++i >= argc) usage();
         depth = atoi(argv[i]);
+      } else if(!strcmp(argv[i], "-mask")){
+        if(++i >= argc) usage();
+        mask = atoi(argv[i]);
       } else {
         usage();
       }
@@ -508,7 +620,7 @@ static int runsearch(int argc, char **argv){
     }
   }
   if(!nurl || depth < 0) usage();
-  rv = procsearch(nurl, cbdatumptr(pbuf), attrs, ord, max, opts, depth, ec);
+  rv = procsearch(nurl, cbdatumptr(pbuf), attrs, ord, max, sk, aux, opts, depth, mask, ec);
   return rv;
 }
 
@@ -916,9 +1028,10 @@ static int procuriid(const char *nurl, const char *uri){
 /* perform the inform command */
 static int procinform(const char *nurl){
   ESTNODE *node;
+  const CBLIST *admins, *users, *links;
   const char *name, *label;
-  int err, dnum, wnum;
-  double size;
+  int i, err, dnum, wnum;
+  double size, ratio;
   err = FALSE;
   node = est_node_new(nurl);
   if(g_pxhost && g_pxport > 0) est_node_set_proxy(node, g_pxhost, g_pxport);
@@ -930,8 +1043,32 @@ static int procinform(const char *nurl){
   dnum = est_node_doc_num(node);
   wnum = est_node_word_num(node);
   size = est_node_size(node);
-  if(name && label && dnum >= 0 && wnum >= 0 && size >= 0.0){
-    printf("%s\t%s\t%d\t%d\t%.0f\n", name, label, dnum, wnum, size);
+  ratio = est_node_cache_usage(node);
+  admins = est_node_admins(node);
+  users = est_node_users(node);
+  links = est_node_links(node);
+  if(name && label && dnum >= 0 && wnum >= 0 && size >= 0.0 && ratio >= 0.0 &&
+     admins && users && links){
+    switch(g_infomode){
+    default:
+      printf("%s\t%s\t%d\t%d\t%.0f\t%.6f\n", name, label, dnum, wnum, size, ratio);
+      break;
+    case IM_ADMINS:
+      for(i = 0; i < cblistnum(admins); i++){
+        printf("%s\n", cblistval(admins, i, NULL));
+      }
+      break;
+    case IM_USERS:
+      for(i = 0; i < cblistnum(users); i++){
+        printf("%s\n", cblistval(users, i, NULL));
+      }
+      break;
+    case IM_LINKS:
+      for(i = 0; i < cblistnum(links); i++){
+        printf("%s\n", cblistval(links, i, NULL));
+      }
+      break;
+    }
   } else {
     printferror("failed: %d", est_node_status(node));
     err = TRUE;
@@ -941,16 +1078,55 @@ static int procinform(const char *nurl){
 }
 
 
+/* perform the sync command */
+static int procsync(const char *nurl){
+  ESTNODE *node;
+  int err;
+  err = FALSE;
+  node = est_node_new(nurl);
+  if(g_pxhost && g_pxport > 0) est_node_set_proxy(node, g_pxhost, g_pxport);
+  if(g_timeout >= 0) est_node_set_timeout(node, g_timeout);
+  if(g_authname && g_authpass) est_node_set_auth(node, g_authname, g_authpass);
+  err = FALSE;
+  if(!est_node_sync(node)){
+    printferror("failed: %d", est_node_status(node));
+    err = TRUE;
+  }
+  est_node_delete(node);
+  return err ? 1 : 0;
+}
+
+
+/* perform the optimize command */
+static int procoptimize(const char *nurl){
+  ESTNODE *node;
+  int err;
+  err = FALSE;
+  node = est_node_new(nurl);
+  if(g_pxhost && g_pxport > 0) est_node_set_proxy(node, g_pxhost, g_pxport);
+  if(g_timeout >= 0) est_node_set_timeout(node, g_timeout);
+  if(g_authname && g_authpass) est_node_set_auth(node, g_authname, g_authpass);
+  err = FALSE;
+  
+  if(!est_node_optimize(node)){
+    printferror("failed: %d", est_node_status(node));
+    err = TRUE;
+  }
+  est_node_delete(node);
+  return err ? 1 : 0;
+}
+
+
 /* perform the inform command */
 static int procsearch(const char *nurl, const char *phrase, const CBLIST *attrs, const char *ord,
-                      int max, int opts, int depth, double ec){
+                      int max, int sk, int aux, int opts, int depth, int mask, double ec){
   ESTNODE *node;
   ESTNODERES *nres;
   ESTRESDOC *rdoc, **sdocs;
   ESTCOND *cond;
   CBMAP *hints;
   CBLIST *names, *elems, *lines;
-  const char *kbuf, *vbuf;
+  const char *kbuf, *vbuf, *ndurl, *ndscore;
   char tbuf[NUMBUFSIZ];
   int i, j, err, ksiz, vsiz, dnum, wnum, snum, id, ld;
   err = FALSE;
@@ -968,12 +1144,35 @@ static int procsearch(const char *nurl, const char *phrase, const CBLIST *attrs,
     est_cond_add_attr(cond, cblistval(attrs, i, NULL));
   }
   if(ord) est_cond_set_order(cond, ord);
-  if(max >= 0) est_cond_set_max(cond, ec >= 0.01 ? max * ((2.0 / ec) + 0.5): max);
+  if(max >= 0){
+    est_cond_set_max(cond, ec >= 0.01 ? max * ((2.0 / ec) + 0.5) : max);
+  } else {
+    max = INT_MAX;
+  }
+  if(sk > 0) est_cond_set_skip(cond, sk);
   est_cond_set_options(cond, opts);
+  est_cond_set_auxiliary(cond, aux);
+  est_cond_set_mask(cond, mask);
   if((nres = est_node_search(node, cond, depth)) != NULL){
     hints = est_noderes_hints(nres);
     if(ec > 0.0) est_noderes_eclipse(nres, max, ec);
     switch(g_viewmode){
+    case VM_URI:
+      printf("%s\n", est_border_str());
+      cbmapiterinit(hints);
+      while((kbuf = cbmapiternext(hints, &ksiz)) != NULL){
+        printf("%s\t%s\n", kbuf, cbmapget(hints, kbuf, ksiz, NULL));
+      }
+      printf("\n");
+      printf("%s\n", est_border_str());
+      for(i = 0; i < max && i < est_noderes_doc_num(nres); i++){
+        rdoc = est_noderes_get_doc(nres, i);
+        ndurl = est_resdoc_attr(rdoc, DATTRNDURL);
+        ndscore = est_resdoc_attr(rdoc, DATTRNDSCORE);
+        printf("%s\t%s\t%s\n", est_resdoc_uri(rdoc), ndurl ? ndurl : "", ndscore ? ndscore : "");
+      }
+      printf("%s:END\n", est_border_str());
+      break;
     case VM_XML:
       xmlprintf("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
       xmlprintf("<estresult xmlns=\"%@\" version=\"%@\">\n", _EST_XNSEARCH, est_version);
@@ -992,8 +1191,12 @@ static int procsearch(const char *nurl, const char *phrase, const CBLIST *attrs,
         }
         cblistclose(elems);
       }
-      if((vbuf = cbmapget(hints, "TIME", -1, NULL)) != NULL)
-        xmlprintf("<time time=\"%@\"/>\n", vbuf);
+      if((vbuf = cbmapget(hints, "TIME", -1, NULL)) != NULL){
+        xmlprintf("<time total=\"%@\"", vbuf);
+        if((vbuf = cbmapget(hints, "TIME#i", -1, NULL)) != NULL)
+          xmlprintf(" index=\"%@\"", vbuf);
+        xmlprintf("/>\n");
+      }
       dnum = 0;
       wnum = 0;
       if((vbuf = cbmapget(hints, "DOCNUM", -1, NULL)) != NULL) dnum = atoi(vbuf);
@@ -1005,11 +1208,15 @@ static int procsearch(const char *nurl, const char *phrase, const CBLIST *attrs,
         elems = cbsplit(vbuf, vsiz, "\t");
         if(cblistnum(elems) == 7){
           xmlprintf("<link url=\"%@\" label=\"%@\" credit=\"%@\" docnum=\"%@\" wordnum=\"%@\""
-                    " size=\"%@\" hit=\"%@\"/>\n",
+                    " size=\"%@\" hit=\"%@\"",
                     cblistval(elems, 0, NULL), cblistval(elems, 1, NULL),
                     cblistval(elems, 2, NULL), cblistval(elems, 3, NULL),
                     cblistval(elems, 4, NULL), cblistval(elems, 5, NULL),
                     cblistval(elems, 6, NULL));
+          sprintf(tbuf, "TIME#%d", i);
+          if((vbuf = cbmapget(hints, tbuf, -1, &vsiz)) != NULL)
+            xmlprintf(" time=\"%@\"", vbuf);
+          xmlprintf("/>\n");
         }
         cblistclose(elems);
       }
@@ -1027,7 +1234,7 @@ static int procsearch(const char *nurl, const char *phrase, const CBLIST *attrs,
                     kbuf, est_resdoc_attr(rdoc, kbuf));
         }
         cblistclose(names);
-        vbuf = est_resdoc_vector(rdoc);
+        vbuf = est_resdoc_keywords(rdoc);
         if(vbuf[0] != '\0'){
           xmlprintf("<vector>");
           elems = cbsplit(vbuf, -1, "\t");
@@ -1084,7 +1291,7 @@ static int procsearch(const char *nurl, const char *phrase, const CBLIST *attrs,
           printf("%s=%s\n", kbuf, est_resdoc_attr(rdoc, kbuf));
         }
         cblistclose(names);
-        vbuf = est_resdoc_vector(rdoc);
+        vbuf = est_resdoc_keywords(rdoc);
         if(vbuf[0] != '\0') printf("%s\t%s\n", ESTDCNTLVECTOR, vbuf);
         sdocs = est_resdoc_shadows(rdoc, &snum);
         for(j = 0; j < snum; j++){
@@ -1219,7 +1426,7 @@ static int procraw(const char *url, const char *file){
   resheads = cbmapopenex(MINIBNUM);
   resbody = cbdatumopen("", 0);
   auth = (g_authname && g_authpass) ? cbsprintf("%s:%s", g_authname, g_authpass) : NULL;
-  if(est_url_shuttle(url, g_pxhost, g_pxport, g_timeout, auth,
+  if(est_url_shuttle(url, g_pxhost, g_pxport, g_timeout, RESLIMSIZE, auth,
                      g_exheaders, reqbody, rbsiz, &rescode, resheads, resbody)){
     if(rescode < 200 && rescode >= 300) err = TRUE;
     if(g_nonparse){
