@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of the node API
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -34,10 +34,11 @@
 #define ESTVAXVECNUM   256               /* maximum number of vector dimensions */
 #define ESTECLPMAX     64                /* maximum number of documents to be eclipsed */
 #define ESTITERUNIT    256               /* retrieval unit number of iterator */
+#define ESTRESLIMSIZE  (1048576*128)     /* limitation of the entity body of response */
 #define ESTWWIDTHDEF   480               /* default of whole width of a snippet */
 #define ESTHWIDTHDEF   96                /* default of head width of a snippet */
 #define ESTAWIDTHDEF   96                /* default of around width of a snippet */
-#define ESTDNHOLDSEC   300               /* holding time of domain names */
+#define ESTDNHOLDSEC   30               /* holding time of domain names */
 #define ESTDNHOLDNUM   4096              /* holding number of domain names */
 
 typedef struct {                         /* type of structure for interaction of a URL */
@@ -46,6 +47,7 @@ typedef struct {                         /* type of structure for interaction of
   const char *url;                       /* URL */
   const char *pxhost;                    /* host name of proxy */
   int pxport;                            /* port number of proxy */
+  int limsize;                           /* limitation of the response body */
   const char *auth;                      /* authority */
   const CBLIST *reqheads;                /* request headers */
   const char *reqbody;                   /* request body */
@@ -57,6 +59,7 @@ typedef struct {                         /* type of structure for interaction of
 
 
 /* private function prototypes */
+static char *est_gethostaddrbyname(const char *name);
 static int est_sock_close(int sock);
 static int est_inet_aton(const char *cp, struct in_addr *inp);
 static void *est_url_shuttle_impl(void *targ);
@@ -74,7 +77,7 @@ static void est_parse_search_body(ESTNODERES *nres, char *str);
  *************************************************************************************************/
 
 
-/* Cache of host addresses */
+/* Cache of host addresses. */
 CBMAP *est_host_addrs = NULL;
 int est_host_attrs_cnt = 0;
 pthread_mutex_t est_host_addrs_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -289,8 +292,8 @@ const char *est_resdoc_snippet(ESTRESDOC *rdoc){
 }
 
 
-/* Get the keyword vector of a result document object. */
-const char *est_resdoc_vector(ESTRESDOC *rdoc){
+/* Get keywords of a result document object. */
+const char *est_resdoc_keywords(ESTRESDOC *rdoc){
   const char *vbuf;
   assert(rdoc);
   vbuf = cbmapget(rdoc->attrs, ESTDCNTLVECTOR, -1, NULL);
@@ -337,6 +340,9 @@ ESTNODE *est_node_new(const char *url){
   node->dnum = -1;
   node->wnum = -1;
   node->size = -1.0;
+  node->admins = NULL;
+  node->users = NULL;
+  node->links = NULL;
   node->wwidth = ESTWWIDTHDEF;
   node->hwidth = ESTHWIDTHDEF;
   node->awidth = ESTAWIDTHDEF;
@@ -353,6 +359,9 @@ void est_node_delete(ESTNODE *node){
   assert(node);
   cbmapclose(node->heads);
   est_node_iter_delete(node);
+  if(node->links) cblistclose(node->links);
+  if(node->users) cblistclose(node->users);
+  if(node->admins) cblistclose(node->admins);
   free(node->label);
   free(node->name);
   free(node->auth);
@@ -393,6 +402,81 @@ int est_node_status(ESTNODE *node){
 }
 
 
+/* Synchronize updating contents of the database of a node. */
+int est_node_sync(ESTNODE *node){
+  CBLIST *reqheads;
+  CBDATUM *reqbody;
+  const char *kbuf;
+  char url[ESTPATHBUFSIZ], *vbuf;
+  int rescode, err, ksiz;
+  assert(node);
+  err = FALSE;
+  sprintf(url, "%s/sync", node->url);
+  reqheads = cblistopen();
+  if(cbmaprnum(node->heads) > 0){
+    cbmapiterinit(node->heads);
+    while((kbuf = cbmapiternext(node->heads, &ksiz)) != NULL){
+      vbuf = cbsprintf("%s: %s", kbuf, cbmapget(node->heads, kbuf, ksiz, NULL));
+      cblistpush(reqheads, vbuf, -1);
+      free(vbuf);
+    }
+  }
+  cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
+  reqbody = cbdatumopen("", 0);
+
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, NULL)){
+    node->status = -1;
+    err = TRUE;
+  }
+  if(!err){
+    node->status = rescode;
+    if(rescode != 200) err = TRUE;
+  }
+  cbdatumclose(reqbody);
+  cblistclose(reqheads);
+  return err ? FALSE : TRUE;
+}
+
+
+/* Optimize the database of a node. */
+int est_node_optimize(ESTNODE *node){
+  CBLIST *reqheads;
+  CBDATUM *reqbody;
+  const char *kbuf;
+  char url[ESTPATHBUFSIZ], *vbuf;
+  int rescode, err, ksiz;
+  assert(node);
+  err = FALSE;
+  sprintf(url, "%s/optimize", node->url);
+  reqheads = cblistopen();
+  if(cbmaprnum(node->heads) > 0){
+    cbmapiterinit(node->heads);
+    while((kbuf = cbmapiternext(node->heads, &ksiz)) != NULL){
+      vbuf = cbsprintf("%s: %s", kbuf, cbmapget(node->heads, kbuf, ksiz, NULL));
+      cblistpush(reqheads, vbuf, -1);
+      free(vbuf);
+    }
+  }
+  cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
+  reqbody = cbdatumopen("", 0);
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, NULL)){
+    node->status = -1;
+    err = TRUE;
+  }
+  if(!err){
+    node->status = rescode;
+    if(rescode != 200) err = TRUE;
+  }
+  cbdatumclose(reqbody);
+  cblistclose(reqheads);
+  return err ? FALSE : TRUE;
+}
+
+
 /* Add a document to a node. */
 int est_node_put_doc(ESTNODE *node, ESTDOC *doc){
   CBLIST *reqheads;
@@ -413,8 +497,8 @@ int est_node_put_doc(ESTNODE *node, ESTDOC *doc){
   }
   cblistpush(reqheads, "Content-Type: " ESTDRAFTTYPE, -1);
   reqbody = est_doc_dump_draft(doc);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      reqbody, strlen(reqbody), &rescode, NULL, NULL)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, reqbody, strlen(reqbody), &rescode, NULL, NULL)){
     node->status = -1;
     err = TRUE;
   }
@@ -450,8 +534,9 @@ int est_node_out_doc(ESTNODE *node, int id){
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen("", 0);
   est_datum_printf(reqbody, "id=%d", id);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, NULL)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, NULL)){
     node->status = -1;
     err = TRUE;
   }
@@ -487,8 +572,9 @@ int est_node_out_doc_by_uri(ESTNODE *node, const char *uri){
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
   reqbody = cbdatumopen("", 0);
   est_datum_printf(reqbody, "uri=%?", uri);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, NULL)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, NULL)){
     node->status = -1;
     err = TRUE;
   }
@@ -532,8 +618,9 @@ int est_node_edit_doc(ESTNODE *node, ESTDOC *doc){
     cbdatumcat(reqbody, "\n", 1);
   }
   cbdatumcat(reqbody, "\n", 1);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, NULL)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, NULL)){
     node->status = -1;
     err = TRUE;
   }
@@ -572,8 +659,9 @@ ESTDOC *est_node_get_doc(ESTNODE *node, int id){
   reqbody = cbdatumopen("", 0);
   est_datum_printf(reqbody, "id=%d", id);
   resbody = cbdatumopen("", 0);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, resbody)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, resbody)){
     node->status = -1;
     err = TRUE;
   }
@@ -613,8 +701,9 @@ ESTDOC *est_node_get_doc_by_uri(ESTNODE *node, const char *uri){
   reqbody = cbdatumopen("", 0);
   est_datum_printf(reqbody, "uri=%?", uri);
   resbody = cbdatumopen("", 0);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, resbody)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, resbody)){
     node->status = -1;
     err = TRUE;
   }
@@ -653,8 +742,9 @@ char *est_node_get_doc_attr(ESTNODE *node, int id, const char *name){
   reqbody = cbdatumopen("", 0);
   est_datum_printf(reqbody, "id=%d&attr=%?", id, name);
   resbody = cbdatumopen("", 0);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, resbody)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, resbody)){
     node->status = -1;
     err = TRUE;
   }
@@ -698,8 +788,9 @@ char *est_node_get_doc_attr_by_uri(ESTNODE *node, const char *uri, const char *n
   reqbody = cbdatumopen("", 0);
   est_datum_printf(reqbody, "uri=%?&attr=%?", uri, name);
   resbody = cbdatumopen("", 0);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, resbody)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, resbody)){
     node->status = -1;
     err = TRUE;
   }
@@ -745,8 +836,9 @@ CBMAP *est_node_etch_doc(ESTNODE *node, int id){
   reqbody = cbdatumopen("", 0);
   est_datum_printf(reqbody, "id=%d", id);
   resbody = cbdatumopen("", 0);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, resbody)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, resbody)){
     node->status = -1;
     err = TRUE;
   }
@@ -798,8 +890,9 @@ CBMAP *est_node_etch_doc_by_uri(ESTNODE *node, const char *uri){
   reqbody = cbdatumopen("", 0);
   est_datum_printf(reqbody, "uri=%?", uri);
   resbody = cbdatumopen("", 0);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, resbody)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, resbody)){
     node->status = -1;
     err = TRUE;
   }
@@ -850,8 +943,9 @@ int est_node_uri_to_id(ESTNODE *node, const char *uri){
   reqbody = cbdatumopen("", 0);
   est_datum_printf(reqbody, "uri=%?", uri);
   resbody = cbdatumopen("", 0);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, resbody)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, resbody)){
     node->status = -1;
     err = TRUE;
   }
@@ -912,13 +1006,78 @@ double est_node_size(ESTNODE *node){
 }
 
 
+/* Get the usage ratio of the cache of a node. */
+double est_node_cache_usage(ESTNODE *node){
+  CBLIST *reqheads;
+  CBDATUM *resbody;
+  const char *kbuf;
+  char url[ESTPATHBUFSIZ], *vbuf;
+  int rescode, err, ksiz;
+  double ratio;
+  assert(node);
+  err = FALSE;
+  sprintf(url, "%s/cacheusage", node->url);
+  reqheads = cblistopen();
+  if(cbmaprnum(node->heads) > 0){
+    cbmapiterinit(node->heads);
+    while((kbuf = cbmapiternext(node->heads, &ksiz)) != NULL){
+      vbuf = cbsprintf("%s: %s", kbuf, cbmapget(node->heads, kbuf, ksiz, NULL));
+      cblistpush(reqheads, vbuf, -1);
+      free(vbuf);
+    }
+  }
+  resbody = cbdatumopen("", 0);
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, NULL, -1, &rescode, NULL, resbody)){
+    node->status = -1;
+    err = TRUE;
+  }
+  if(!err){
+    node->status = rescode;
+    if(rescode != 200) err = TRUE;
+  }
+  ratio = err ? -1.0 : strtod(cbdatumptr(resbody), NULL);
+  cbdatumclose(resbody);
+  cblistclose(reqheads);
+  return ratio;
+}
+
+
+/* Get a list of names of administrators of a node. */
+const CBLIST *est_node_admins(ESTNODE *node){
+  assert(node);
+  if(node->admins) return node->admins;
+  est_node_set_info(node);
+  return node->admins;
+}
+
+
+/* Get a list of names of users of a node. */
+const CBLIST *est_node_users(ESTNODE *node){
+  assert(node);
+  if(node->users) return node->users;
+  est_node_set_info(node);
+  return node->users;
+}
+
+
+/* Get a list of expressions of links of a node. */
+const CBLIST *est_node_links(ESTNODE *node){
+  assert(node);
+  if(node->links) return node->links;
+  est_node_set_info(node);
+  return node->links;
+}
+
+
 /* Search documents corresponding a condition for a node. */
 ESTNODERES *est_node_search(ESTNODE *node, ESTCOND *cond, int depth){
   ESTNODERES *nres;
   const CBLIST *attrs;
+  CBMAP *resheads;
   CBLIST *reqheads;
   CBDATUM *reqbody, *resbody;
-  const char *kbuf, *phrase, *order;
+  const char *kbuf, *phrase, *order, *cbuf;
   char buf[ESTPATHBUFSIZ], *vbuf, *ptr, *pv, *ep;
   int i, rescode, err, ksiz, max, plen, part, end;
   assert(node && cond && depth >= 0);
@@ -955,13 +1114,18 @@ ESTNODERES *est_node_search(ESTNODE *node, ESTCOND *cond, int depth){
   }
   if(cbdatumsize(reqbody) > 0) cbdatumcat(reqbody, "&", 1);
   est_datum_printf(reqbody, "options=%d", est_cond_options(cond));
+  est_datum_printf(reqbody, "&auxiliary=%d", est_cond_auxiliary(cond));
   est_datum_printf(reqbody, "&depth=%d", depth);
   est_datum_printf(reqbody, "&wwidth=%d", node->wwidth);
   est_datum_printf(reqbody, "&hwidth=%d", node->hwidth);
   est_datum_printf(reqbody, "&awidth=%d", node->awidth);
+  est_datum_printf(reqbody, "&skip=%d", est_cond_skip(cond));
+  est_datum_printf(reqbody, "&mask=%d", est_cond_mask(cond));
+  resheads = cbmapopenex(ESTMINIBNUM);
   resbody = cbdatumopen("", 0);
-  if(!est_url_shuttle(buf, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, resbody)){
+  if(!est_url_shuttle(buf, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, resheads, resbody)){
     node->status = -1;
     err = TRUE;
   }
@@ -971,6 +1135,7 @@ ESTNODERES *est_node_search(ESTNODE *node, ESTCOND *cond, int depth){
   }
   if(err){
     cbdatumclose(resbody);
+    cbmapclose(resheads);
     cbdatumclose(reqbody);
     cblistclose(reqheads);
     return NULL;
@@ -1013,6 +1178,12 @@ ESTNODERES *est_node_search(ESTNODE *node, ESTCOND *cond, int depth){
     }
   }
   free(ptr);
+  if((cbuf = cbmapget(resheads, "last-modified", -1, NULL)) != NULL){
+    nres->mdate = cbstrmktime(cbuf);
+  } else {
+    nres->mdate = time(NULL);
+  }
+  cbmapclose(resheads);
   cbdatumclose(reqbody);
   cblistclose(reqheads);
   if(!end){
@@ -1055,8 +1226,9 @@ int est_node_set_user(ESTNODE *node, const char *name, int mode){
   reqbody = cbdatumopen("", 0);
   est_datum_printf(reqbody, "name=%?&mode=%d", name, mode);
   resbody = cbdatumopen("", 0);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, resbody)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, resbody)){
     node->status = -1;
     err = TRUE;
   }
@@ -1098,8 +1270,9 @@ int est_node_set_link(ESTNODE *node, const char *url, const char *label, int cre
     est_datum_printf(reqbody, "url=%?&label=%?", url, label);
   }
   resbody = cbdatumopen("", 0);
-  if(!est_url_shuttle(myurl, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, resbody)){
+  if(!est_url_shuttle(myurl, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, resbody)){
     node->status = -1;
     err = TRUE;
   }
@@ -1136,7 +1309,6 @@ const char *est_get_host_name(void){
 
 /* Get the address of a host. */
 char *est_get_host_addr(const char *name){
-  struct hostent *info;
   const char *addr;
   char *buf, *pv, vbuf[64];
   int i, ost, nsiz, asiz, vsiz;
@@ -1171,15 +1343,10 @@ char *est_get_host_addr(const char *name){
     free(buf);
     return NULL;
   }
-  if((info = gethostbyname(name)) != NULL && info->h_addr_list[0]){
-    addr = inet_ntoa(*(struct in_addr *)info->h_addr_list[0]);
-  } else {
-    addr = NULL;
-  }
-  buf = addr ? cbmemdup(addr, -1) : NULL;
+  buf = est_gethostaddrbyname(name);
   if(pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &ost) == 0){
     if(pthread_mutex_lock(&est_host_addrs_mutex) == 0){
-      vsiz = sprintf(vbuf, "%s\t%d", addr ? addr : "", (int)time(NULL));
+      vsiz = sprintf(vbuf, "%s\t%d", buf ? buf : "", (int)time(NULL));
       cbmapput(est_host_addrs, name, nsiz, vbuf, vsiz, TRUE);
       if(cbmaprnum(est_host_addrs) > ESTDNHOLDNUM){
         cbmapiterinit(est_host_addrs);
@@ -1466,7 +1633,7 @@ void est_datum_printf(CBDATUM *datum, const char *format, ...){
 
 
 /* Perform an interaction of a URL. */
-int est_url_shuttle(const char *url, const char *pxhost, int pxport, int outsec,
+int est_url_shuttle(const char *url, const char *pxhost, int pxport, int outsec, int limsize,
                     const char *auth, const CBLIST *reqheads, const char *reqbody, int rbsiz,
                     int *rescodep, CBMAP *resheads, CBDATUM *resbody){
   pthread_t th;
@@ -1483,6 +1650,7 @@ int est_url_shuttle(const char *url, const char *pxhost, int pxport, int outsec,
   targ.url = url;
   targ.pxhost = pxhost;
   targ.pxport = pxport;
+  targ.limsize = limsize;
   targ.auth = auth;
   targ.reqheads = reqheads;
   targ.reqbody = reqbody;
@@ -1577,6 +1745,7 @@ ESTNODERES *est_noderes_new(void){
   nres->survivors = NULL;
   nres->snum = 0;
   nres->hints = cbmapopenex(ESTMINIBNUM);
+  nres->mdate = 0;
   return nres;
 }
 
@@ -1621,12 +1790,58 @@ int est_noderes_shift_doc(ESTNODERES *nres, CBMAP **attrp, char **snippetp){
 }
 
 
+/* Get the modification date of a node result object. */
+time_t est_noderes_mdate(ESTNODERES *nres){
+  assert(nres);
+  return nres->mdate;
+}
+
+
 
 /*************************************************************************************************
  * private objects
  *************************************************************************************************/
 
 
+/* Get the primary address of a host name.
+   `name' specifies the name of a host.
+   The return value is the address of a host or `NULL' if failure.   Because the region of the
+   return value is allocated with the `malloc' call, it should be released with the `free' call
+   if it is no longer in use. */
+static char *est_gethostaddrbyname(const char *name){
+#if defined(_SYS_LINUX_)
+  struct hostent infobuf, *info;
+  char buf[ESTIOBUFSIZ], *addr;
+  int ecode;
+  if(gethostbyname_r(name, &infobuf, buf, ESTIOBUFSIZ, &info, &ecode) == 0 && info){
+    addr = cbmemdup(inet_ntoa(*(struct in_addr *)info->h_addr_list[0]), -1);
+  } else {
+    addr = NULL;
+  }
+  return addr;
+#else
+  static pthread_mutex_t mymutex = PTHREAD_MUTEX_INITIALIZER;
+  struct hostent *info;
+  char *addr;
+  int ost;
+  assert(name);
+  if(pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, &ost) != 0) return NULL;
+  if(pthread_mutex_lock(&mymutex) != 0){
+    pthread_setcancelstate(ost, NULL);
+    return NULL;
+  }
+  if((info = gethostbyname(name)) != NULL && info->h_addr_list[0]){
+    addr = cbmemdup(inet_ntoa(*(struct in_addr *)info->h_addr_list[0]), -1);
+  } else {
+    addr = NULL;
+  }
+  pthread_mutex_unlock(&mymutex);
+  pthread_setcancelstate(ost, NULL);
+  return addr;
+#endif
+}
+
+
 /* Close a socket.
    `sock' specifies a socket.
    The return value is 0 if success, else it is -1. */
@@ -1674,11 +1889,12 @@ static void *est_url_shuttle_impl(void *targ){
   CBDATUM *resbody, *datum;
   const char *url, *pxhost, *auth, *reqbody, *tmp, *scheme, *host, *path, *query, *rp;
   char *addr, *enc, iobuf[ESTIOBUFSIZ], name[ESTIOBUFSIZ], *pv, *zbuf;
-  int i, pxport, rbsiz, *rescodep, port, sock, *sp, size, nsiz, defl, zsiz;
+  int i, pxport, limsize, rbsiz, *rescodep, port, sock, *sp, size, nsiz, defl, zsiz;
   assert(targ);
   url = ((TARGSHUTTLE *)targ)->url;
   pxhost = ((TARGSHUTTLE *)targ)->pxhost;
   pxport = ((TARGSHUTTLE *)targ)->pxport;
+  limsize = ((TARGSHUTTLE *)targ)->limsize;
   auth = ((TARGSHUTTLE *)targ)->auth;
   reqheads = ((TARGSHUTTLE *)targ)->reqheads;
   reqbody = ((TARGSHUTTLE *)targ)->reqbody;
@@ -1785,13 +2001,12 @@ static void *est_url_shuttle_impl(void *targ){
     }
     if(cbstrfwimatch(iobuf, "content-encoding:") && strstr(iobuf + 17, "deflate")) defl = TRUE;
   }
+  if(limsize < 0) limsize = INT_MAX;
   while((size = recv(sock, iobuf, ESTIOBUFSIZ, 0)) > 0){
-    if(resbody) cbdatumcat(resbody, iobuf, size);
+    if(resbody && cbdatumsize(resbody) <= limsize) cbdatumcat(resbody, iobuf, size);
   }
-  if(defl && (zbuf = est_inflate(cbdatumptr(resbody), cbdatumsize(resbody), &zsiz)) != NULL){
-    cbdatumsetsize(resbody, 0);
-    cbdatumcat(resbody, zbuf, zsiz);
-    free(zbuf);
+  if(defl && (zbuf = est_inflate(cbdatumptr(resbody), cbdatumsize(resbody), &zsiz, 0)) != NULL){
+    cbdatumsetbuf(resbody, zbuf, zsiz);
   }
   pthread_cleanup_pop(1);
   pthread_cleanup_pop(1);
@@ -1817,9 +2032,9 @@ static void est_sockpt_down(void *sp){
 static int est_node_set_info(ESTNODE *node){
   CBLIST *reqheads, *elems;
   CBDATUM *resbody;
-  const char *kbuf, *ptr;
-  char url[ESTPATHBUFSIZ], *vbuf, *pv;
-  int rescode, err, ksiz;
+  const char *kbuf, *ptr, *pv, *tv;
+  char url[ESTPATHBUFSIZ], *vbuf;
+  int rescode, err, ksiz, tc;
   assert(node);
   err = FALSE;
   sprintf(url, "%s/inform", node->url);
@@ -1835,9 +2050,21 @@ static int est_node_set_info(ESTNODE *node){
   node->dnum = -1;
   node->wnum = -1;
   node->size = -1.0;
+  if(node->admins){
+    cblistclose(node->admins);
+    node->admins = NULL;
+  }
+  if(node->users){
+    cblistclose(node->users);
+    node->users = NULL;
+  }
+  if(node->links){
+    cblistclose(node->links);
+    node->links = NULL;
+  }
   resbody = cbdatumopen("", 0);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      NULL, -1, &rescode, NULL, resbody)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, NULL, -1, &rescode, NULL, resbody)){
     node->status = -1;
     err = TRUE;
   }
@@ -1867,6 +2094,44 @@ static int est_node_set_info(ESTNODE *node){
           node->size = -1.0;
           err = TRUE;
         }
+        ptr = pv;
+        ptr++;
+        if(*ptr == '\n') ptr++;
+        node->admins = cblistopen();
+        while((pv = strchr(ptr, '\n')) != NULL){
+          if(ptr == pv){
+            ptr = pv + 1;
+            break;
+          }
+          cblistpush(node->admins, ptr, pv - ptr);
+          ptr = pv + 1;
+        }
+        if(*pv == '\n') pv++;
+        node->users = cblistopen();
+        while((pv = strchr(ptr, '\n')) != NULL){
+          if(ptr == pv){
+            ptr = pv + 1;
+            break;
+          }
+          cblistpush(node->users, ptr, pv - ptr);
+          ptr = pv + 1;
+        }
+        if(*pv == '\n') pv++;
+        node->links = cblistopen();
+        while((pv = strchr(ptr, '\n')) != NULL){
+          if(ptr == pv){
+            ptr = pv + 1;
+            break;
+          }
+          tc = 0;
+          tv = ptr;
+          while(tv < pv){
+            if(*tv == '\t') tc++;
+            tv++;
+          }
+          if(tc == 2) cblistpush(node->links, ptr, pv - ptr);
+          ptr = pv + 1;
+        }
       } else {
         err = TRUE;
       }
@@ -1925,8 +2190,9 @@ static int est_node_read_list(ESTNODE *node){
   est_datum_printf(reqbody, "max=%d", ESTITERUNIT);
   if(node->iprev) est_datum_printf(reqbody, "&prev=%?", node->iprev);
   resbody = cbdatumopen("", 0);
-  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, node->auth, reqheads,
-                      cbdatumptr(reqbody), cbdatumsize(reqbody), &rescode, NULL, resbody)){
+  if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
+                      reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
+                      &rescode, NULL, resbody)){
     node->status = -1;
     err = TRUE;
   }
