@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The core API of Hyper Estraier
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -66,12 +66,14 @@ extern const char *est_version;
 #define ESTDATTRWEIGHT "@weight"         /* name of the attribute of scoring weight */
 #define ESTDATTRMISC   "@misc"           /* name of the attribute of miscellaneous information */
 #define ESTDCNTLVECTOR "%VECTOR"         /* name of the control code for keyword vector */
+#define ESTDCNTLSCORE  "%SCORE"          /* name of the control code for substitute score */
 #define ESTDCNTLSHADOW "%SHADOW"         /* name of the control code for shadow document */
 
 typedef struct {                         /* type of structure for a document */
   int id;                                /* identification number */
   CBMAP *attrs;                          /* map of attributes */
   CBLIST *dtexts;                        /* list of shown text */
+  CBMAP *kwords;                         /* map of keywords */
 } ESTDOC;
 
 
@@ -110,6 +112,20 @@ void est_doc_add_text(ESTDOC *doc, const char *text);
 void est_doc_add_hidden_text(ESTDOC *doc, const char *text);
 
 
+/* Attache keywords to a document object.
+   `doc' specifies a document object.
+   `kwords' specifies a map object of keywords.  Keys of the map should be keywords of the
+   document and values should be their scores in decimal string.  The map object is copied
+   internally. */
+void est_doc_set_keywords(ESTDOC *doc, CBMAP *kwords);
+
+
+/* Set the substitute score of a document object.
+   `doc' specifies a document object.
+   `score' specifies the substitute score.  It should be zero or positive. */
+void est_doc_set_score(ESTDOC *doc, int score);
+
+
 /* Get the ID number of a document object.
    `doc' specifies a document object.
    The return value is the ID number of the document object.  If the object has not been
@@ -148,6 +164,14 @@ const CBLIST *est_doc_texts(ESTDOC *doc);
 char *est_doc_cat_texts(ESTDOC *doc);
 
 
+/* Get attached keywords of a document object.
+   `doc' specifies a document object.
+   The return value is a map object of keywords and their scores in decimal string.  If no
+   keyword is attached, `NULL' is returned.  The life duration of the returned object is
+   synchronous with the one of the document object. */
+CBMAP *est_doc_keywords(ESTDOC *doc);
+
+
 /* Dump draft data of a document object.
    `doc' specifies a document object.
    The return value is draft data of the document object.  Because the region of the return value
@@ -197,6 +221,7 @@ char *est_doc_make_snippet(ESTDOC *doc, const CBLIST *words, int wwidth, int hwi
 #define ESTOPSTREW     "STREW"           /* string ends with */
 #define ESTOPSTRAND    "STRAND"          /* string includes all tokens in */
 #define ESTOPSTROR     "STROR"           /* string includes at least one token in */
+#define ESTOPSTROREQ   "STROREQ"         /* string is equal at least one token in */
 #define ESTOPSTRRX     "STRRX"           /* string matches regular expressions of */
 #define ESTOPNUMEQ     "NUMEQ"           /* number or date is equal */
 #define ESTOPNUMNE     "NUMNE"           /* number or date is not equal */
@@ -206,25 +231,39 @@ char *est_doc_make_snippet(ESTDOC *doc, const CBLIST *words, int wwidth, int hwi
 #define ESTOPNUMLE     "NUMLE"           /* number or date is less than or equal to */
 #define ESTOPNUMBT     "NUMBT"           /* number or date is between two tokens of */
 
+#define ESTORDIDA     "[IDA]"            /* ID numbers in ascending order */
+#define ESTORDIDD     "[IDD]"            /* ID numbers in descending order */
+#define ESTORDSCA     "[SCA]"            /* scores in ascending order */
+#define ESTORDSCD     "[SCD]"            /* scores in descending order */
 #define ESTORDSTRA     "STRA"            /* strings in ascending order */
 #define ESTORDSTRD     "STRD"            /* strings in descending order */
 #define ESTORDNUMA     "NUMA"            /* numbers in ascending order */
 #define ESTORDNUMD     "NUMD"            /* numbers in descending order */
 
+#define ESTECLSIMURL   10.0              /* eclipse considering similarity and URL */
+#define ESTECLSERV     100.0             /* eclipse on server basis */
+#define ESTECLDIR      101.0             /* eclipse on directory basis */
+#define ESTECLFILE     102.0             /* eclipse on file basis */
+
 typedef struct {                         /* type of structure for search conditions */
   char *phrase;                          /* search phrase */
   int gstep;                             /* step of N-gram */
   int tfidf;                             /* whether with TF-IDF tuning */
   int simple;                            /* whether with the simplified phrase */
+  void (*cbxpn)(const char *, CBLIST *); /* callback function for query expansion */
   CBLIST *attrs;                         /* conditions with attributes */
   char *order;                           /* sorting order */
   int max;                               /* maximum number of retrieval */
+  int skip;                              /* number of documents to be skipped */
+  int auxmin;                            /* minimum hits to adopt the auxiliary index */
+  CBMAP *auxwords;                       /* words which the auxiliary index has been used */
   int scfb;                              /* whether to feed back scores */
   int *scores;                           /* array of scores */
   int snum;                              /* number of elemnts of the score array */
   int opts;                              /* options for preservation */
   double ecllim;                         /* lower limit of similarity eclipse */
   CBMAP *shadows;                        /* map of eclipsed documents */
+  int mask;                              /* mask for meta search */
 } ESTCOND;
 
 enum {                                   /* enumeration for options */
@@ -234,7 +273,6 @@ enum {                                   /* enumeration for options */
   ESTCONDAGITO = 1 << 3,                 /* check N-gram keys skipping by three */
   ESTCONDNOIDF = 1 << 4,                 /* without TF-IDF tuning */
   ESTCONDSIMPLE = 1 << 10,               /* with the simplified phrase */
-  ESTCONDETCH = 1 << 20,                 /* attach keywords (for the node API) */
   ESTCONDSCFB = 1 << 30                  /* feed back scores (for debug) */
 };
 
@@ -274,6 +312,12 @@ void est_cond_set_order(ESTCOND *cond, const char *expr);
 void est_cond_set_max(ESTCOND *cond, int max);
 
 
+/* Set the number of skipped documents of a condition object.
+   `cond' specifies a condition object.
+   `skip' specifies the number of documents to be skipped in the search result. */
+void est_cond_set_skip(ESTCOND *cond, int skip);
+
+
 /* Set options of retrieval of a condition object.
    `cond' specifies a condition object.
    `options' specifies options: `ESTCONDSURE' specifies that it checks every N-gram key,
@@ -287,13 +331,31 @@ void est_cond_set_max(ESTCOND *cond, int max);
 void est_cond_set_options(ESTCOND *cond, int options);
 
 
+/* Set permission to adopt result of the auxiliary index.
+   `cond' specifies a condition object.
+   `min' specifies the minimum hits to adopt result of the auxiliary index.  If it is not more
+   than 0, the auxiliary index is not used.  By default, it is 32. */
+void est_cond_set_auxiliary(ESTCOND *cond, int min);
+
+
 /* Set the lower limit of similarity eclipse.
    `cond' specifies a condition object.
    `limit' specifies the lower limit of similarity for documents to be eclipsed.  Similarity is
-   between 0.0 and 1.0. */
+   between 0.0 and 1.0.  If the limit is added by `ESTECLSIMURL', similarity is weighted by URL.
+   If the limit is `ESTECLSERV', similarity is ignored and documents in the same server are
+   eclipsed.  If the limit is `ESTECLDIR', similarity is ignored and documents in the same
+   directory are eclipsed.  If the limit is `ESTECLFILE', similarity is ignored and documents of
+   the same file are eclipsed. */
 void est_cond_set_eclipse(ESTCOND *cond, double limit);
 
 
+/* Set the mask of targets of meta search.
+   `cond' specifies a condition object.
+   `mask' specifies a masking number.  1 means the first target, 2 means the second target, 4
+   means the third target, and power values of 2 and their summation compose the mask. */
+void est_cond_set_mask(ESTCOND *cond, int mask);
+
+
 
 /*************************************************************************************************
  * API for database
@@ -313,19 +375,26 @@ typedef struct {                         /* type of structure for the inverted i
 
 typedef struct {                         /* type of structure for a database object */
   char *name;                            /* name of the database */
+  int inode;                             /* inode of the database */
   DEPOT *metadb;                         /* handle of the meta database */
   ESTIDX *idxdb;                         /* handles of the inverted indexs */
   VILLA *fwmdb;                          /* handle of the database for forward matching */
+  VILLA *auxdb;                          /* handle of the auxiliary index */
+  VILLA *xfmdb;                          /* handle of the database for aux forward matching */
   CURIA *attrdb;                         /* handle of the database for attrutes */
   CURIA *textdb;                         /* handle of the database for texts */
   CURIA *kwddb;                          /* handle of the database for keywords */
   VILLA *listdb;                         /* handle of the database for document list */
+  CBMAP *aidxs;                          /* map of attribute indexes */
   int ecode;                             /* last happened error code */
   int fatal;                             /* whether to have a fatal error */
   int dseq;                              /* sequence for document IDs */
   int dnum;                              /* number of the documents */
   int amode;                             /* mode of text analyzer */
+  int zmode;                             /* mode of data compression */
+  int smode;                             /* mode of score type */
   CBMAP *idxcc;                          /* cache for the inverted index */
+  CBMAP *auxcc;                          /* cache for the auxiliary index */
   size_t icsiz;                          /* power of the cache */
   size_t icmax;                          /* max size of the cache */
   CBMAP *outcc;                          /* cache for deleted documents */
@@ -342,9 +411,12 @@ typedef struct {                         /* type of structure for a database obj
   CBMAP *spacc;                          /* special cache for attributes */
   int scmnum;                            /* max number of the special cache */
   char *scname;                          /* name of the attribute for the special cache */
-  void (*cbinfo)(const char *);          /* callback function to inform of events */
+  void (*infocb)(const char *, void *);  /* callback function to inform of events */
+  void *infoop;                          /* opaque for the informing callback */
   DEPOT *dfdb;                           /* handle of the database for document frequency */
+  int wildmax;                           /* maximum number of expansion of wild cards */
   CBMAP *metacc;                         /* cache for meta data */
+  int flsflag;                           /* flag of flushing */
   int intflag;                           /* flag of thread interruption */
 } ESTDB;
 
@@ -366,7 +438,19 @@ enum {                                   /* enumeration for open modes */
   ESTDBTRUNC = 1 << 3,                   /* a writer truncating */
   ESTDBNOLCK = 1 << 4,                   /* open without locking */
   ESTDBLCKNB = 1 << 5,                   /* lock without blocking */
-  ESTDBPERFNG = 1 << 6                   /* use perfect N-gram analyzer */
+  ESTDBPERFNG = 1 << 10,                 /* use perfect N-gram analyzer */
+  ESTDBCHRCAT = 1 << 11,                 /* use character category analyzer */
+  ESTDBLARGE = 1 << 20,                  /* large tuning */
+  ESTDBHUGE = 1 << 21,                   /* huge tuning */
+  ESTDBSCVOID = 1 << 25,                 /* store scores as void */
+  ESTDBSCINT = 1 << 26,                  /* store scores as integer */
+  ESTDBSCASIS = 1 << 27                  /* refrain from adjustment of scores */
+};
+
+enum {                                   /* enumeration for data types of attribute index */
+  ESTIDXATTRSEQ,                         /* for multipurpose sequencial access method */
+  ESTIDXATTRSTR,                         /* for narrowing with attributes as strings */
+  ESTIDXATTRNUM                          /* for narrowing with attributes as numbers */
 };
 
 enum {                                   /* enumeration for options of optimization */
@@ -374,8 +458,13 @@ enum {                                   /* enumeration for options of optimizat
   ESTOPTNODBOPT = 1 << 1                 /* omit optimization of the database files */
 };
 
+enum {                                   /* enumeration for options of document merger */
+  ESTMGCLEAN = 1 << 0                    /* clean up dispensable regions */
+};
+
 enum {                                   /* enumeration for options of document registration */
-  ESTPDCLEAN = 1 << 0                    /* clean up dispensable regions */
+  ESTPDCLEAN = 1 << 0,                   /* clean up dispensable regions */
+  ESTPDWEIGHT = 1 << 1                   /* weight scores statically when indexing */
 };
 
 enum {                                   /* enumeration for options of document deletion */
@@ -384,7 +473,8 @@ enum {                                   /* enumeration for options of document
 
 enum {                                   /* enumeration for options of document retrieval */
   ESTGDNOATTR = 1 << 0,                  /* no attributes */
-  ESTGDNOTEXT = 1 << 1                   /* no text */
+  ESTGDNOTEXT = 1 << 1,                  /* no text */
+  ESTGDNOKWD = 1 << 2                    /* no keywords */
 };
 
 
@@ -403,7 +493,13 @@ const char *est_err_msg(int ecode);
    bitwise or: `ESTDBNOLCK', which means it opens a database file without file locking, or
    `ESTDBLCKNB', which means locking is performed without blocking.  If `ESTDBNOLCK' is used,
    the application is responsible for exclusion control.  `ESTDBCREAT' can be added to by bitwise
-   or: `ESTDBPERFNG', which means N-gram analysis is performed against European text also.
+   or: `ESTDBPERFNG', which means N-gram analysis is performed against European text also,
+   `ESTDBCHACAT', which means character category analysis is performed instead of N-gram analysis,
+   `ESTDBLARGE', which means the index is tuned to register more than 300000 documents,
+   `ESTDBHUGE', which means the index is tuned to register more than 1000000 documents,
+   `ESTDBSCVOID', which means scores are stored as void, `ESTDBSCINT', which means scores are
+   stored as 32-bit integer, `ESTDBSCASIS', which means scores are stored as-is and marked not
+   to be tuned when search.
    `ecp' specifies the pointer to a variable to which the error code is assigned.
    The return value is a database object of the database or `NULL' if failure. */
 ESTDB *est_db_open(const char *name, int omode, int *ecp);
@@ -428,6 +524,17 @@ int est_db_error(ESTDB *db);
 int est_db_fatal(ESTDB *db);
 
 
+/* Add an index for narrowing or sorting with document attributes.
+   `db' specifies a database object connected as a writer.
+   `name' specifies the name of an attribute.
+   `type' specifies the data type of attribute index; `ESTIDXATTRSEQ' for multipurpose sequencial
+   access method, `ESTIDXATTRSTR' for narrowing with attributes as strings, `ESTIDXATTRNUM' for
+   narrowing with attributes as numbers.
+   The return value is true if success, else it is false.
+   Note that this function should be called before the first document is registered. */
+int est_db_add_attr_index(ESTDB *db, const char *name, int type);
+
+
 /* Flush index words in the cache of a database.
    `db' specifies a database object connected as a writer.
    `max' specifies the maximum number of words to be flushed.  If it not more than zero, all
@@ -451,11 +558,23 @@ int est_db_sync(ESTDB *db);
 int est_db_optimize(ESTDB *db, int options);
 
 
+/* Merge another database.
+   `db' specifies a database object connected as a writer.
+   `name' specifies the name of another database directory.
+   `options' specifies options: `ESTMGCLEAN' to clean up dispensable regions of the deleted
+   document.
+   The return value is true if success, else it is false.
+   Creation options of the two databases should be same entirely.  ID numbers of imported
+   documents are changed within the sequence of the desitination database.  If URIs of imported
+   documents conflict ones of exsisting documents, existing documents are removed. */
+int est_db_merge(ESTDB *db, const char *name, int options);
+
+
 /* Add a document to a database.
    `db' specifies a database object connected as a writer.
    `doc' specifies a document object.  The document object should have the URI attribute.
    `options' specifies options: `ESTPDCLEAN' to clean up dispensable regions of the overwritten
-   document.
+   document, `ESTPDWEIGHT' to weight scores statically with score weighting attribute.
    The return value is true if success, else it is false.
    If the URI attribute is same with an existing document in the database, the existing one is
    deleted. */
@@ -484,8 +603,10 @@ int est_db_edit_doc(ESTDB *db, ESTDOC *doc);
    `db' specifies a database object.
    `id' specifies the ID number of a registered document.
    `options' specifies options: `ESTGDNOATTR' to ignore attributes, `ESTGDNOTEXT' to ignore
-   the body text.  The two can be specified at the same time by bitwise or.
-   The return value is a document object.  On error, `NULL' is returned. */
+   the body text, `ESTGDNOKWD' to ignore keywords.  The three can be specified at the same time
+   by bitwise or.
+   The return value is a document object.  It should be deleted with `est_doc_delete' if it is
+   no longer in use.  On error, `NULL' is returned. */
 ESTDOC *est_db_get_doc(ESTDB *db, int id, int options);
 
 
@@ -531,7 +652,7 @@ int est_db_word_num(ESTDB *db);
 double est_db_size(ESTDB *db);
 
 
-/* Search documents corresponding a condition for a database.
+/* Search a database for documents corresponding a condition.
    `db' specifies a database object.
    `cond' specifies a condition object.
    `nump' specifies the pointer to a variable to which the number of elements in the result is
@@ -546,6 +667,23 @@ double est_db_size(ESTDB *db);
 int *est_db_search(ESTDB *db, ESTCOND *cond, int *nump, CBMAP *hints);
 
 
+/* Search plural databases for documents corresponding a condition.
+   `dbs' specifies an array whose elements are database objects.
+   `dbnum' specifies the number of elements of the array.
+   `cond' specifies a condition object.
+   `nump' specifies the pointer to a variable to which the number of elements in the result is
+   assigned.
+   `hints' specifies a map object into which the number of documents corresponding to each word
+   is stored.  If a word is in a negative condition, the number is negative.  The element whose
+   key is an empty string specifies the number of whole result.  If it is `NULL', it is not used.
+   The return value is an array whose elements are indexes of container databases and ID numbers
+   of in each database alternately.
+   This function does never fail.  Even if no document corresponds or an error occurs, an empty
+   array is returned.  Because the region of the return value is allocated with the `malloc'
+   call, it should be released with the `free' call if it is no longer in use. */
+int *est_db_search_meta(ESTDB **dbs, int dbnum, ESTCOND *cond, int *nump, CBMAP *hints);
+
+
 /* Check whether a document object matches the phrase of a search condition object definitely.
    `db' specifies a database object.
    `doc' specifies a document object.
@@ -574,8 +712,8 @@ void est_db_set_cache_size(ESTDB *db, size_t size, int anum, int tnum, int rnum)
  *************************************************************************************************/
 
 
-#define _EST_VERSION   "1.0.6"
-#define _EST_LIBVER    700
+#define _EST_VERSION   "1.3.3"
+#define _EST_LIBVER    818
 #define _EST_PROTVER   "1.0"
 
 #define _EST_PROJURL   "http://hyperestraier.sourceforge.net/"
@@ -596,6 +734,11 @@ enum {                                   /* enumeration for document parts */
   ESTMDKWD = 1 << 2                      /* keywords */
 };
 
+enum {                                   /* enumeration for database repair */
+  ESTRPSTRICT = 1 << 0,                  /* perform strict consistency check */
+  ESTRPSHODDY = 1 << 1                   /* omit consistency check */
+};
+
 
 /* Break a sentence of text and extract words.
    `text' specifies a sentence of text.
@@ -613,6 +756,13 @@ void est_break_text(const char *text, CBLIST *list, int norm, int tail);
 void est_break_text_perfng(const char *text, CBLIST *list, int norm, int tail);
 
 
+/* Break a sentence of text and extract words, using character category analyzer.
+   `text' specifies a sentence of text.
+   `list' specifies a list object to which extract words are added.
+   `norm' specifies whether to normalize the text. */
+void est_break_text_chrcat(const char *text, CBLIST *list, int norm);
+
+
 /* Convert the character encoding of a string.
    `ptr' specifies the pointer to a region.
    `size' specifies the size of the region.  If it is negative, the size is assigned with
@@ -672,13 +822,42 @@ char *est_uconv_out(const char *ptr, int size, int *sp);
    `strlen(ptr)'.
    `sp' specifies the pointer to a variable to which the size of the region of the return
    value is assigned.
+   `mode' specifies detail behavior.  0 specifies using the standard deflate encoding, -1
+   specifies the raw deflate encoding, and 1 specifies the GZIP encoding.
    If successful, the return value is the pointer to the result object, else, it is `NULL'.
    Because the region of the return value is allocated with the `malloc' call, it should be
    released with the `free' call if it is no longer in use. */
-char *est_deflate(const char *ptr, int size, int *sp);
+char *est_deflate(const char *ptr, int size, int *sp, int mode);
 
 
 /* Decompress a serial object compressed with ZLIB.
+   `ptr' specifies the pointer to a region.
+   `size' specifies the size of the region.
+   `sp' specifies the pointer to a variable to which the size of the region of the return
+   value is assigned.  If it is `NULL', it is not used.
+   `mode' specifies detail behavior.  0 specifies using the standard deflate encoding, -1
+   specifies the raw deflate encoding, and 1 specifies the GZIP encoding.
+   If successful, the return value is the pointer to the result object, else, it is `NULL'.
+   Because an additional zero code is appended at the end of the region of the return value,
+   the return value can be treated as a character string.  Because the region of the return
+   value is allocated with the `malloc' call, it should be released with the `free' call if it
+   is no longer in use. */
+char *est_inflate(const char *ptr, int size, int *sp, int mode);
+
+
+/* Compress a serial object with LZO.
+   `ptr' specifies the pointer to a region.
+   `size' specifies the size of the region.  If it is negative, the size is assigned with
+   `strlen(ptr)'.
+   `sp' specifies the pointer to a variable to which the size of the region of the return
+   value is assigned.
+   If successful, the return value is the pointer to the result object, else, it is `NULL'.
+   Because the region of the return value is allocated with the `malloc' call, it should be
+   released with the `free' call if it is no longer in use. */
+char *est_lzoencode(const char *ptr, int size, int *sp);
+
+
+/* Decompress a serial object compressed with LZO.
    `ptr' specifies the pointer to a region.
    `size' specifies the size of the region.
    `sp' specifies the pointer to a variable to which the size of the region of the return
@@ -688,7 +867,32 @@ char *est_deflate(const char *ptr, int size, int *sp);
    the return value can be treated as a character string.  Because the region of the return
    value is allocated with the `malloc' call, it should be released with the `free' call if it
    is no longer in use. */
-char *est_inflate(const char *ptr, int size, int *sp);
+char *est_lzodecode(const char *ptr, int size, int *sp);
+
+
+/* Compress a serial object with BZIP2.
+   `ptr' specifies the pointer to a region.
+   `size' specifies the size of the region.  If it is negative, the size is assigned with
+   `strlen(ptr)'.
+   `sp' specifies the pointer to a variable to which the size of the region of the return
+   value is assigned.
+   If successful, the return value is the pointer to the result object, else, it is `NULL'.
+   Because the region of the return value is allocated with the `malloc' call, it should be
+   released with the `free' call if it is no longer in use. */
+char *est_bzencode(const char *ptr, int size, int *sp);
+
+
+/* Decompress a serial object compressed with BZIP2.
+   `ptr' specifies the pointer to a region.
+   `size' specifies the size of the region.
+   `sp' specifies the pointer to a variable to which the size of the region of the return
+   value is assigned.  If it is `NULL', it is not used.
+   If successful, the return value is the pointer to the result object, else, it is `NULL'.
+   Because an additional zero code is appended at the end of the region of the return value,
+   the return value can be treated as a character string.  Because the region of the return
+   value is allocated with the `malloc' call, it should be released with the `free' call if it
+   is no longer in use. */
+char *est_bzdecode(const char *ptr, int size, int *sp);
 
 
 /* Get the border string for draft data of documents.
@@ -723,7 +927,8 @@ int est_match_crypt(const char *key, const char *hash);
 
 /* Create a regular expression object.
    `str' specifies a string of regular expressions.
-   The return value is a regular expression object or `NULL' if failure. */
+   The return value is a regular expression object or `NULL' if failure.
+   If the expression is leaded by "*I:", the pattern is case insensitive. */
 void *est_regex_new(const char *str);
 
 
@@ -742,13 +947,20 @@ int est_regex_match(const void *regex, const char *str);
 /* Replace each substring matching a regular expression string.
    `str' specifies a target string.
    `bef' specifies a string of regular expressions for substrings.
-   `aft' specifies a string with which each substrings are replaced.
+   `aft' specifies a string with which each substrings are replaced.  Each "&" in the string is
+   replaced with the matched substring.  Each "\" in the string escapes the following character.
    The return value is a new converted string.  Even if the regular expression is invalid, a copy
    of the original string is returned.  Because the region of the return value is allocated with
    the `malloc' call, it should be released with the `free' call if it is no longer in use. */
 char *est_regex_replace(const char *str, const char *bef, const char *aft);
 
 
+/* Duplicate a document object.
+   `doc' specifies a document object.
+   The return value is a duplicated document object. */
+ESTDOC *est_doc_dup(ESTDOC *doc);
+
+
 /* Set the ID number of a document object.
    `doc' specifies a document object.
    `id' specifies the ID number to set. */
@@ -762,12 +974,24 @@ void est_doc_set_id(ESTDOC *doc, int id);
 const char *est_doc_hidden_texts(ESTDOC *doc);
 
 
+/* Reduce the texts to fit to the specified size.
+   `doc' specifies a document object.
+   `len' specifies the total size of the texts. */
+void est_doc_slim(ESTDOC *doc, int size);
+
+
 /* Check whether a docuemnt object is empty.
    `doc' specifies a document object.
    The return value is true the document is empty, else it is false. */
 int est_doc_is_empty(ESTDOC *doc);
 
 
+/* Duplicate a condition object.
+   `cond' specifies a condition object.
+   The return value is a duplicated condition object. */
+ESTCOND *est_cond_dup(ESTCOND *cond);
+
+
 /* Get the phrase of a condition object.
    `cond' specifies a condition object.
    The return value is the phrase of the condition object or `NULL' if it is not specified.  The
@@ -798,12 +1022,30 @@ const char *est_cond_order(ESTCOND *cond);
 int est_cond_max(ESTCOND *cond);
 
 
+/* Get the number of skipped documents of a condition object.
+   `cond' specifies a condition object.
+   The return value is the number of documents to be skipped in the search result. */
+int est_cond_skip(ESTCOND *cond);
+
+
 /* Get the options of a condition object.
    `cond' specifies a condition object.
    The return value is the options of the condition object. */
 int est_cond_options(ESTCOND *cond);
 
 
+/* Get permission to adopt result of the auxiliary index.
+   `cond' specifies a condition object.
+   The return value is permission to adopt result of the auxiliary index. */
+int est_cond_auxiliary(ESTCOND *cond);
+
+
+/* Get the mask of targets of meta search.
+   `cond' specifies a condition object.
+   The return value is the mask of targets of meta search. */
+int est_cond_mask(ESTCOND *cond);
+
+
 /* Get the score of a document corresponding to a condition object.
    `cond' specifies a condition object.
    `index' specifies the index of an element of the result array of `est_db_search'.
@@ -811,9 +1053,18 @@ int est_cond_options(ESTCOND *cond);
 int est_cond_score(ESTCOND *cond, int index);
 
 
+/* Check whether a condition object has used the auxiliary index.
+   `cond' specifies a condition object.
+   `word' specifies a keyword to be checked.  If it is an empty string, whether at least one
+   keyword is used is checked.
+   The return value is true if the condition object has used the auxiliary index, else it is
+   false */
+int est_cond_auxiliary_word(ESTCOND *cond, const char *word);
+
+
 /* Get an array of ID numbers of eclipsed docuemnts of a document in a condition object.
    `cond' specifies a condition object.
-   `id' specifies the ID number of the parent document.
+   `id' specifies the ID number of a parent document.
    `np' specifies the pointer to a variable to which the number of elements of the return value
    is assigned.
    The return value is an array whose elements expresse the ID numbers and their scores
@@ -821,12 +1072,26 @@ int est_cond_score(ESTCOND *cond, int index);
 const int *est_cond_shadows(ESTCOND *cond, int id, int *np);
 
 
+/* Set the callback function for query expansion.
+   `cond' specifies a condition object.
+   `func' specifies the pointer to a function.  The first argument of the callback specifies a
+   word to be expand.  The second argument speciifes a list object into which renewed words to
+   be stored. */
+void est_cond_set_expander(ESTCOND *cond, void (*func)(const char *, CBLIST *));
+
+
 /* Set the error code of a database.
    `db' specifies a database object.
    `ecode' specifies a error code to set. */
 void est_db_set_ecode(ESTDB *db, int ecode);
 
 
+/* Get the inode number of a database.
+   `db' specifies a database object.
+   The return value is the inode number of the database. */
+int est_db_inode(ESTDB *db);
+
+
 /* Set the entity data of a document in a database.
    `db' specifies a database object connected as a writer.
    `id' specifies the ID number of a registered document.
@@ -848,6 +1113,12 @@ int est_db_set_doc_entity(ESTDB *db, int id, const char *ptr, int size);
 char *est_db_get_doc_entity(ESTDB *db, int id, int *sp);
 
 
+/* Set the maximum number of expansion of wild cards.
+   `db' specifies a database object.
+   `num' specifies the maximum number of expansion of wild cards. */
+void est_db_set_wildmax(ESTDB *db, int num);
+
+
 /* Add a piece of meta data to a database.
    `db' specifies a database object connected as a writer.
    `name' specifies the name of a piece of meta data.
@@ -886,8 +1157,16 @@ CBMAP *est_db_etch_doc(ESTDB *db, ESTDOC *doc, int max);
    `db' specifies a database object connected as a writer.
    `id' specifies the ID number of a document.
    `kwords' specifies a map object of keywords of the document.
+   `weight' specifies weighting bias of scores.
    The return value is true if success, else it is false. */
-int est_db_put_keywords(ESTDB *db, int id, CBMAP *kwords);
+int est_db_put_keywords(ESTDB *db, int id, CBMAP *kwords, double weight);
+
+
+/* Remove keywords of a document.
+   `db' specifies a database object connected as a writer.
+   `id' specifies the ID number of a document.
+   The return value is true if success, else it is false. */
+int est_db_out_keywords(ESTDB *db, int id);
 
 
 /* Retrieve a map object of keywords.
@@ -945,6 +1224,46 @@ char *est_db_word_iter_next(ESTDB *db);
 int est_db_word_rec_size(ESTDB *db, const char *word);
 
 
+/* Get the number of unique keywords in a database.
+   `db' specifies a database object.
+   The return value is the number of unique keywords in the database. */
+int est_db_keyword_num(ESTDB *db);
+
+
+/* Initialize the keyword iterator of a database.
+   `db' specifies a database object.
+   The return value is true if success, else it is false. */
+int est_db_keyword_iter_init(ESTDB *db);
+
+
+/* Get the next keyword of the word iterator of a database.
+   `db' specifies a database object.
+   The return value is the next word.  If there is no more keyword, `NULL' is returned.  Because
+   the region of the return value is allocated with the `malloc' call, it should be released
+   with the `free' call if it is no longer in use. */
+char *est_db_keyword_iter_next(ESTDB *db);
+
+
+/* Get the size of the record of a keyword.
+   `db' specifies a database object.
+   `word' specifies a keyword.
+   The return value is the size of the record of the keyword.  If there is no corresponding
+   record, 0 is returned. */
+int est_db_keyword_rec_size(ESTDB *db, const char *word);
+
+
+/* Search documents corresponding a keyword for a database.
+   `db' specifies a database object.
+   `word' specifies a keyword.
+   `nump' specifies the pointer to a variable to which the number of elements in the result is
+   assigned.
+   The return value is an array whose elements are ID numbers of corresponding documents.
+   This function does never fail.  Even if no document corresponds or an error occurs, an empty
+   array is returned.  Because the region of the return value is allocated with the `malloc'
+   call, it should be released with the `free' call if it is no longer in use. */
+int *est_db_keyword_search(ESTDB *db, const char *word, int *nump);
+
+
 /* Get the number of records in the cache memory of a database.
    `db' specifies a database object.
    The return value is the cache memory of a database. */
@@ -966,9 +1285,10 @@ void est_db_set_special_cache(ESTDB *db, const char *name, int num);
 
 /* Set the callback function to inform of database events.
    `db' specifies a database object.
-   `func' specifies the pointer to a function.  The argument of the callback specifies a message
-   of each event. */
-void est_db_set_informer(ESTDB *db, void (*func)(const char *));
+   `func' specifies the pointer to a function.  The first argument of the callback specifies a
+   message of each event.  The second argument specifies an arbitrary pointer of a opaque data.
+   `opaque' specifies the pointer of the second argument of the callback. */
+void est_db_set_informer(ESTDB *db, void (*func)(const char *, void *), void *opaque);
 
 
 /* Fill the cache for keys for TF-IDF.
@@ -1007,6 +1327,15 @@ CBLIST *est_db_list_rescc(ESTDB *db);
 void est_db_interrupt(ESTDB *db);
 
 
+/* Repair a broken database directory.
+   `name' specifies the name of a database directory.
+   `options' specifies options: `ESTRPSTRICT' to perform strict consistency check, `ESTRPSHODDY'
+   to omit consistency check.
+   `ecp' specifies the pointer to a variable to which the error code is assigned.
+   The return value is true if success, else it is false. */
+int est_db_repair(const char *name, int options, int *ecp);
+
+
 /* Extract words for snippet from hints of search.
    `hints' specifies a map object whose records were set by `est_db_search'.
    The return value is a new list object of words to be highlighted.  Because the object of the
@@ -1015,6 +1344,12 @@ void est_db_interrupt(ESTDB *db);
 CBLIST *est_hints_to_words(CBMAP *hints);
 
 
+/* Reset the environment of the process.
+   This function sets the standard streams as binary mode and resets environment variables for
+   locale. */
+void est_proc_env_reset(void);
+
+
 /* Make a directory.
    `path' specifies the path of a new directory.
    The return value is true if success, else it is false. */
@@ -1028,13 +1363,19 @@ int est_rmdir_rec(const char *path);
 
 
 /* Get the canonicalized absolute pathname of a file.
-   `path' specifies the path of a new directory.
+   `path' specifies the path of a file.
    The return value is the canonicalized absolute pathname of a file.  Because the region of the
    return value is allocated with the `malloc' call, it should be released with the `free' call
    if it is no longer in use. */
 char *est_realpath(const char *path);
 
 
+/* Get the inode number of a file.
+   `path' specifies the path of a file.
+   The return value is the inode number of a file or -1 on error. */
+int est_inode(const char *path);
+
+
 /* Get the time of day in milliseconds.
    The return value is the time of day in milliseconds. */
 double est_gettimeofday(void);
@@ -1059,6 +1400,12 @@ void est_signal(int signum, void (*sighandler)(int));
 int est_kill(int pid, int sig);
 
 
+/* Get the load ratio of the physical memory.
+   The return value is the load ratio of the physical memory.
+   As for now, this function returns 0.0 on platforms except for Windows. */
+double est_memory_usage(void);
+
+
 /* Get the media type of an extention.
    `ext' specifies the extension of a file path.
    The return value is the media time of the extension. */
