@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * A sample searcher of Hyper Estraier
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -34,6 +34,11 @@
 #define NAVIPAGES      10                /* number of pages in paging navigation */
 #define SPCACHEMNUM    1048576           /* max number of the special cache */
 
+typedef struct {                         /* type of structure for a hitting object */
+  const char *word;                      /* face of keyword */
+  int pt;                                /* score tuned by TF-IDF */
+} KEYSC;
+
 
 /* global variables for configurations */
 const char *g_conffile = NULL;           /* path of the configuration file */
@@ -41,9 +46,9 @@ const char *g_indexname = NULL;          /* name of the index */
 const char *g_tmplfile = NULL;           /* path of the template file */
 const char *g_topfile = NULL;            /* path of the top page file */
 const char *g_logfile = NULL;            /* path of the log file */
+int g_lockindex = FALSE;                 /* whether to perform file locking to the database */
 const CBLIST *g_replexprs = NULL;        /* list of URI replacement expressions */
 int g_showlreal = FALSE;                 /* wether to show local real paths */
-const char *g_perpage = NULL;            /* CSV of numbers of shown documents per page */
 int g_attrselect = FALSE;                /* whether to use select boxes for extension form */
 int g_showscore = FALSE;                 /* whether to show scores */
 const CBLIST *g_extattrs = NULL;         /* list of extra attributes of each document */
@@ -55,9 +60,15 @@ int g_dotfidf = FALSE;                   /* whether to do TF-IDF tuning */
 int g_scancheck = FALSE;                 /* whether to check documents by scanning */
 int g_smplphrase = FALSE;                /* whether to use simplified phrase */
 int g_candetail = FALSE;                 /* whether to show detail link */
+int g_auxmin = -1;                       /* minimum hits to adopt the auxiliary index */
 int g_smlrvnum = -1;                     /* number of elements of a vecter for similarity */
+const char *g_smlrtune = NULL;           /* tuning parameters for similarity search */
 int g_clipview = -1;                     /* number of clipped documents to be shown */
+double g_clipweight = 0.0;               /* weighting algorithm of documents clipping */
+int g_relkeynum = -1;                    /* number of related keywords to be shown */
 const char *g_spcache = NULL;            /* name of the attribute of special cache */
+int g_wildmax = -1;                      /* maximum number of extension of wild cards */
+const char *g_qxpndcmd = NULL;           /* command for query expansion */
 
 
 /* global variables for parameters */
@@ -67,7 +78,9 @@ const char *p_attrval = NULL;            /* separated value of narrowing attribu
 const char *p_order = NULL;              /* ordering attribute */
 int p_perpage = 0;                       /* number of show documents per page */
 int p_clip = 0;                          /* lower limit of similarity eclipse */
-int p_cinc = FALSE;                      /* whether to include clipped documents */
+int p_qxpnd = FALSE;                     /* whether to perform query expansion */
+int p_cinc = 0;                          /* ID of the parent of shown clipped documents */
+int p_prec = FALSE;                      /* whether to search more precisely */
 int p_pagenum = 0;                       /* number of the page */
 int p_detail = 0;                        /* ID of the document to be detailed */
 int p_similar = 0;                       /* ID of the seed document of similarity search */
@@ -95,7 +108,10 @@ static void setsimilarphrase(void);
 static void showpage(void);
 static void showform(void);
 static void showtop(void);
-static void showresult(ESTDOC **docs, int dnum, CBMAP *hints, ESTCOND *cond, int miss);
+static void expandquery(const char *word, CBLIST *result);
+static int keysc_compare(const void *ap, const void *bp);
+static void showresult(ESTDOC **docs, int dnum, CBMAP *hints, ESTCOND *cond, int hits, int miss,
+                       KEYSC *scores, int scnum);
 static void showdoc(ESTDOC *doc, const CBLIST *words, CBMAP *cnames, int detail,
                     const int *shadows, int snum, int *clipp);
 static char *makeshownuri(const char *uri);
@@ -119,7 +135,9 @@ int main(int argc, char **argv){
     p_order = NULL;
     p_perpage = 0;
     p_clip = 0;
-    p_cinc = FALSE;
+    p_qxpnd = FALSE;
+    p_cinc = 0;
+    p_prec = FALSE;
     p_pagenum = 0;
     p_detail = 0;
     p_similar = 0;
@@ -140,7 +158,7 @@ static int realmain(int argc, char **argv){
   char *tmp, *wp;
   int i, ecode;
   /* set configurations */
-  cbstdiobin();
+  est_proc_env_reset();
   setvbuf(stdout, g_outbuf, _IOFBF, OUTBUFSIZ);
   g_scriptname = argv[0];
   if((rp = getenv("SCRIPT_NAME")) != NULL) g_scriptname = rp;
@@ -167,12 +185,12 @@ static int realmain(int argc, char **argv){
       g_topfile = skiplabel(rp);
     } else if(cbstrfwimatch(rp, "logfile:")){
       g_logfile = skiplabel(rp);
+    } else if(cbstrfwimatch(rp, "lockindex:")){
+      if(!cbstricmp(skiplabel(rp), "true")) g_lockindex = TRUE;
     } else if(cbstrfwimatch(rp, "replace:")){
       cblistpush(rlist, skiplabel(rp), -1);
     } else if(cbstrfwimatch(rp, "showlreal:")){
       if(!cbstricmp(skiplabel(rp), "true")) g_showlreal = TRUE;
-    } else if(cbstrfwimatch(rp, "perpage:")){
-      g_perpage = skiplabel(rp);
     } else if(cbstrfwimatch(rp, "attrselect:")){
       if(!cbstricmp(skiplabel(rp), "true")) g_attrselect = TRUE;
     } else if(cbstrfwimatch(rp, "showscore:")){
@@ -195,12 +213,25 @@ static int realmain(int argc, char **argv){
       if(!cbstricmp(skiplabel(rp), "true")) g_smplphrase = TRUE;
     } else if(cbstrfwimatch(rp, "candetail:")){
       if(!cbstricmp(skiplabel(rp), "true")) g_candetail = TRUE;
+    } else if(cbstrfwimatch(rp, "auxmin:")){
+      g_auxmin = atoi(skiplabel(rp));
     } else if(cbstrfwimatch(rp, "smlrvnum:")){
       g_smlrvnum = atoi(skiplabel(rp));
+    } else if(cbstrfwimatch(rp, "smlrtune:")){
+      g_smlrtune = skiplabel(rp);
     } else if(cbstrfwimatch(rp, "clipview:")){
       g_clipview = atoi(skiplabel(rp));
+    } else if(cbstrfwimatch(rp, "clipweight:")){
+      rp = skiplabel(rp);
+      if(!cbstricmp(rp, "url")) g_clipweight = ESTECLSIMURL;
+    } else if(cbstrfwimatch(rp, "relkeynum:")){
+      g_relkeynum = atoi(skiplabel(rp));
     } else if(cbstrfwimatch(rp, "spcache:")){
       g_spcache = skiplabel(rp);
+    } else if(cbstrfwimatch(rp, "wildmax:")){
+      g_wildmax = atoi(skiplabel(rp));
+    } else if(cbstrfwimatch(rp, "qxpndcmd:")){
+      g_qxpndcmd = skiplabel(rp);
     }
   }
   if(!g_indexname) showerror("indexname is undefined.");
@@ -208,13 +239,15 @@ static int realmain(int argc, char **argv){
   if(!g_topfile) showerror("topfile is undefined.");
   if(!g_logfile) showerror("logfile is undefined.");
   g_replexprs = rlist;
-  if(!g_perpage) showerror("perpage is undefined.");
   g_extattrs = alist;
   if(g_snipwwidth < 0) showerror("snipwwidth is undefined.");
   if(g_sniphwidth < 0) showerror("sniphwidth is undefined.");
   if(g_snipawidth < 0) showerror("snipawidth is undefined.");
   if(g_condgstep < 1) showerror("condgstep is undefined.");
+  if(!g_smlrtune) showerror("smlrtune is undefined.");
   if(!g_spcache) showerror("spcache is undefined.");
+  if(g_wildmax < 0) showerror("wildmax is undefined.");
+  if(!g_qxpndcmd) showerror("qxpndcmd is undefined.");
   /* read parameters */
   params = getparameters();
   cbglobalgc(params, (void (*)(void *))cbmapclose);
@@ -249,7 +282,9 @@ static int realmain(int argc, char **argv){
   if((rp = cbmapget(params, "perpage", -1, NULL)) != NULL) p_perpage = atoi(rp);
   if(p_perpage < 1) p_perpage = DEFPERPAGE;
   if((rp = cbmapget(params, "clip", -1, NULL)) != NULL) p_clip = atoi(rp);
-  if((rp = cbmapget(params, "cinc", -1, NULL)) != NULL) p_cinc = atoi(rp) > 0;
+  if((rp = cbmapget(params, "qxpnd", -1, NULL)) != NULL) p_qxpnd = atoi(rp);
+  if((rp = cbmapget(params, "cinc", -1, NULL)) != NULL) p_cinc = atoi(rp);
+  if((rp = cbmapget(params, "prec", -1, NULL)) != NULL) p_prec = atoi(rp) > 0;
   if((rp = cbmapget(params, "detail", -1, NULL)) != NULL) p_detail = atoi(rp);
   if(p_detail < 1) p_detail = 0;
   if((rp = cbmapget(params, "similar", -1, NULL)) != NULL) p_similar = atoi(rp);
@@ -283,13 +318,15 @@ static int realmain(int argc, char **argv){
     cbglobalgc(tmp, free);
     g_toptext = tmp;
     for(i = 0; i <= LOCKRETRYNUM; i++){
-      if((g_db = est_db_open(g_indexname, ESTDBREADER | ESTDBLCKNB, &ecode)) != NULL) break;
+      if((g_db = est_db_open(g_indexname, ESTDBREADER | (g_lockindex ? ESTDBLCKNB : ESTDBNOLCK),
+                             &ecode)) != NULL) break;
       if(ecode != ESTELOCK) showerror("the index is missing or broken.");
       est_usleep(1000 * 1000);
     }
     if(!g_db) showerror("the index is being updated now.");
     cbglobalgc(g_db, (void (*)(void *))myestdbclose);
     if(g_spcache[0] != '\0') est_db_set_special_cache(g_db, g_spcache, SPCACHEMNUM);
+    est_db_set_wildmax(g_db, g_wildmax);
   }
   setsimilarphrase();
   /* show the page */
@@ -497,7 +534,8 @@ static void setsimilarphrase(void){
   } else if(!svmap){
     return;
   }
-  datum = cbdatumopen(ESTOPSIMILAR, -1);
+  datum = cbdatumopen(ESTOPSIMILAR " ", -1);
+  cbdatumcat(datum, g_smlrtune, -1);
   cbmapiterinit(svmap);
   while((kbuf = cbmapiternext(svmap, &ksiz)) != NULL){
     vbuf = cbmapget(svmap, kbuf, ksiz, &vsiz);
@@ -515,13 +553,14 @@ static void setsimilarphrase(void){
 
 /* show the page */
 static void showpage(void){
+  KEYSC *scores;
   ESTCOND *cond;
   ESTDOC **docs;
-  CBMAP *hints;
+  CBMAP *hints, *allkwd, *dockwd;
   CBLIST *elems;
-  const char *rp;
+  const char *rp, *vbuf;
   char *tmp, numbuf[NUMBUFSIZ];
-  int i, tnum, max, *res, rnum, sc, dnum, miss;
+  int i, tnum, max, *res, rnum, hits, sc, dnum, miss, len, scnum;
   printf("Cache-Control: no-cache, must-revalidate, no-transform\r\n");
   printf("Pragma: no-cache\r\n");
   printf("Content-Disposition: inline; filename=%s\r\n", g_scriptname);
@@ -529,6 +568,7 @@ static void showpage(void){
   printf("\r\n");
   g_etime = est_gettimeofday();
   cond = est_cond_new();
+  if(g_qxpndcmd[0] != '\0' && p_qxpnd) est_cond_set_expander(cond, expandquery);
   if(p_phrase[0] != '\0') est_cond_set_phrase(cond, p_phrase);
   if(p_attr[0] != '\0'){
     if(p_attrval[0] != '\0'){
@@ -557,13 +597,17 @@ static void showpage(void){
   if(!g_dotfidf) est_cond_set_options(cond, ESTCONDNOIDF);
   if(g_smplphrase) est_cond_set_options(cond, ESTCONDSIMPLE);
   if(g_showscore) est_cond_set_options(cond, ESTCONDSCFB);
-  if(p_clip > 0) est_cond_set_eclipse(cond, p_clip / 10.0);
+  est_cond_set_auxiliary(cond, p_prec ? -1 :
+                         (g_auxmin > p_perpage * 1.3 + 1 ? g_auxmin : p_perpage * 1.3 + 1));
+  if(p_clip > 0) est_cond_set_eclipse(cond, p_clip / 10.0 + (p_clip <= 10 ? g_clipweight: 0));
   tnum = 0;
   max = p_pagenum * p_perpage * 1.3 + 1;
+  hits = 0;
   do {
     est_cond_set_max(cond, max);
     hints = cbmapopenex(MINIBNUM);
     res = est_db_search(g_db, cond, &rnum, hints);
+    hits = (rp = cbmapget(hints, "", 0, NULL)) ? atoi(rp) : rnum;
     if(g_candetail && p_detail > 0){
       if(rnum < 1) cbmapput(hints, "", 0, "1", 1, TRUE);
       free(res);
@@ -576,7 +620,7 @@ static void showpage(void){
     miss = 0;
     for(i = 0; i < rnum; i++){
       if(!(docs[dnum] = est_db_get_doc(g_db, res[i], dnum < p_pagenum * p_perpage || g_scancheck ?
-                                       0 : ESTGDNOATTR | ESTGDNOTEXT))){
+                                       0 : ESTGDNOATTR | ESTGDNOTEXT | ESTGDNOKWD))){
         miss++;
         continue;
       }
@@ -591,7 +635,9 @@ static void showpage(void){
       }
       dnum++;
     }
-    if(tnum <= MISSRETRYNUM && miss > 0 && max <= rnum && dnum < p_pagenum * p_perpage + 1){
+    if((tnum <= MISSRETRYNUM && miss > 0 && max <= rnum && dnum < p_pagenum * p_perpage + 1) ||
+       (p_pagenum == 1 && tnum == 0 && (hits < g_auxmin || hits < p_perpage) &&
+        est_cond_auxiliary_word(cond, ""))){
       for(i = 0; i < dnum; i++){
         est_doc_delete(docs[i]);
       }
@@ -599,11 +645,38 @@ static void showpage(void){
       free(res);
       cbmapclose(hints);
       max *= MISSINCRATIO;
+      if(p_pagenum == 1) est_cond_set_auxiliary(cond, -1);
       tnum++;
       continue;
     }
     break;
   } while(TRUE);
+  if(g_relkeynum > 0){
+    allkwd = cbmapopenex(MINIBNUM);
+    for(i = 0; i < dnum; i++){
+      if(!(dockwd = est_doc_keywords(docs[i]))) continue;
+      cbmapiterinit(dockwd);
+      while((rp = cbmapiternext(dockwd, &len)) != NULL){
+        sc = ((vbuf = cbmapget(allkwd, rp, len, NULL)) != NULL ? atoi(vbuf) : 0) +
+          pow((atoi(cbmapget(dockwd, rp, len, NULL)) + 100) * 10, 0.7);
+        sprintf(numbuf, "%d", sc);
+        cbmapput(allkwd, rp, len, numbuf, -1, TRUE);
+      }
+    }
+    scores = cbmalloc(cbmaprnum(allkwd) * sizeof(KEYSC) + 1);
+    scnum = 0;
+    cbmapiterinit(allkwd);
+    while((rp = cbmapiternext(allkwd, &len)) != NULL){
+      scores[scnum].word = rp;
+      scores[scnum].pt = atoi(cbmapget(allkwd, rp, len, NULL));
+      scnum++;
+    }
+    qsort(scores, scnum, sizeof(KEYSC), keysc_compare);
+  } else {
+    allkwd = NULL;
+    scores = NULL;
+    scnum = 0;
+  }
   g_etime = est_gettimeofday() - g_etime;
   elems = cbxmlbreak(g_tmpltext, FALSE);
   for(i = 0; i < cblistnum(elems); i++){
@@ -614,7 +687,7 @@ static void showpage(void){
       if(p_phrase[0] == '\0' && p_attr[0] == '\0' && p_detail < 1){
         showtop();
       } else {
-        showresult(docs, dnum, hints, cond, miss);
+        showresult(docs, dnum, hints, cond, hits, miss, scores, scnum);
       }
     } else if(!strcmp(rp, "<!--ESTINFO-->")){
       showinfo();
@@ -626,6 +699,8 @@ static void showpage(void){
     est_doc_delete(docs[i]);
   }
   cblistclose(elems);
+  if(scores) free(scores);
+  if(allkwd) cbmapclose(allkwd);
   free(docs);
   free(res);
   cbmapclose(hints);
@@ -635,9 +710,7 @@ static void showpage(void){
 
 /* show the form */
 static void showform(void){
-  CBLIST *list;
-  const char *elem;
-  int i, num;
+  int i;
   xmlprintf("<div id=\"estform\" class=\"estform\">\n");
   xmlprintf("<form action=\"%@\" method=\"get\" id=\"form_self\">\n", g_scriptname);
   xmlprintf("<div class=\"form_basic\">\n");
@@ -650,15 +723,10 @@ static void showform(void){
   xmlprintf("</div>\n");
   xmlprintf("<div class=\"form_extension\">\n");
   xmlprintf("<select name=\"perpage\" id=\"perpage\" tabindex=\"%d\">\n", ++g_tabidx);
-  list = cbsplit(g_perpage, -1, ",");
-  for(i = 0; i < cblistnum(list); i++){
-    elem = cblistval(list, i, NULL);
-    if(elem[0] == '\0') continue;
-    num = atoi(elem);
+  for(i = 10; i <= 100; i += 10){
     xmlprintf("<option value=\"%d\"%s>%d</option>\n",
-              num, num == p_perpage ? " selected=\"selected\"" : "", num);
+              i, i == p_perpage ? " selected=\"selected\"" : "", i);
   }
-  cblistclose(list);
   xmlprintf("</select>\n");
   xmlprintf("per page,\n");
   if(g_attrselect){
@@ -687,7 +755,7 @@ static void showform(void){
               cbstrfwmatch(p_attr, "@size NUMGE") ? " selected=\"selected\"" : "");
     xmlprintf("</select>\n");
     xmlprintf("<input type=\"text\" name=\"attrval\" value=\"%@\""
-              " size=\"16\" id=\"attrval\" class=\"text\" tabindex=\"%d\" accesskey=\"2\" />\n",
+              " size=\"12\" id=\"attrval\" class=\"text\" tabindex=\"%d\" accesskey=\"2\" />\n",
               p_attrval, ++g_tabidx);
     xmlprintf(", order by\n");
     xmlprintf("<select name=\"order\" id=\"order\" tabindex=\"%d\">\n", ++g_tabidx);
@@ -712,11 +780,11 @@ static void showform(void){
   } else {
     xmlprintf("with\n");
     xmlprintf("<input type=\"text\" name=\"attr\" value=\"%@\""
-              " size=\"20\" id=\"attr\" class=\"text\" tabindex=\"%d\" accesskey=\"2\" />\n",
+              " size=\"18\" id=\"attr\" class=\"text\" tabindex=\"%d\" accesskey=\"2\" />\n",
               p_attr, ++g_tabidx);
     xmlprintf(", order by\n");
     xmlprintf("<input type=\"text\" name=\"order\" value=\"%@\""
-              " size=\"16\" id=\"order\" class=\"text\" tabindex=\"%d\" accesskey=\"3\" />\n",
+              " size=\"14\" id=\"order\" class=\"text\" tabindex=\"%d\" accesskey=\"3\" />\n",
               p_order, ++g_tabidx);
   }
   if(g_clipview >= 0){
@@ -727,8 +795,20 @@ static void showform(void){
       xmlprintf("<option value=\"%d\"%s>0.%d</option>\n",
                 i, i == p_clip ? " selected=\"selected\"" : "", i);
     }
+    xmlprintf("<option value=\"%d\"%s>file</option>\n", (int)(ESTECLFILE * 10),
+              p_clip == (int)(ESTECLFILE * 10) ? " selected=\"selected\"" : "", i);
+    xmlprintf("<option value=\"%d\"%s>dir</option>\n", (int)(ESTECLDIR * 10),
+              p_clip == (int)(ESTECLDIR * 10) ? " selected=\"selected\"" : "", i);
+    xmlprintf("<option value=\"%d\"%s>serv</option>\n", (int)(ESTECLSERV * 10),
+              p_clip == (int)(ESTECLSERV * 10) ? " selected=\"selected\"" : "", i);
     xmlprintf("</select>\n");
   }
+  if(g_qxpndcmd[0] != '\0'){
+    xmlprintf(", expansion:\n");
+    xmlprintf("<input type=\"checkbox\" name=\"qxpnd\" value=\"1\"%s id=\"qxpnd\""
+              " class=\"checkbox\" tabindex=\"%d\" accesskey=\"4\" />\n",
+              p_qxpnd ? " checked=\"checked\"" : "", ++g_tabidx);
+  }
   xmlprintf("</div>\n");
   xmlprintf("</form>\n");
   xmlprintf("</div>\n");
@@ -741,16 +821,50 @@ static void showtop(void){
 }
 
 
+/* perform query expansion */
+static void expandquery(const char *word, CBLIST *result){
+  CBLIST *words;
+  const char *tmpdir;
+  char oname[PATH_MAX], cmd[PATH_MAX], *ebuf;
+  int i;
+  cblistpush(result, word, -1);
+  tmpdir = getenv("TMP");
+  if(!tmpdir) tmpdir = getenv("TEMP");
+  if(!tmpdir) tmpdir = ESTPATHSTR "tmp";
+  sprintf(oname, "%s%c%s.%08d", tmpdir, ESTPATHCHR, g_scriptname, (int)getpid());
+  sprintf(cmd, "%s > %s", g_qxpndcmd, oname);
+  ebuf = cbsprintf("ESTWORD=%s", word);
+  putenv(ebuf);
+  system(cmd);
+  free(ebuf);
+  if((words = cbreadlines(oname)) != NULL){
+    for(i = 0; i < cblistnum(words); i++){
+      word = cblistval(words, i, NULL);
+      if(word[0] != '\0') cblistpush(result, word, -1);
+    }
+    cblistclose(words);
+  }
+  unlink(oname);
+}
+
+
+/* compare two keywords by scores in descending order */
+static int keysc_compare(const void *ap, const void *bp){
+  return ((KEYSC *)bp)->pt - ((KEYSC *)ap)->pt;
+}
+
+
 /* show the result */
-static void showresult(ESTDOC **docs, int dnum, CBMAP *hints, ESTCOND *cond, int miss){
+static void showresult(ESTDOC **docs, int dnum, CBMAP *hints, ESTCOND *cond, int hits, int miss,
+                       KEYSC *scores, int scnum){
   CBMAP *cnames;
   CBLIST *words;
   const char *kbuf, *myphrase;
   char cname[NUMBUFSIZ];
   const int *ary;
-  int i, hits, ksiz, snum, start, end, cnum, clip, anum, pnum;
+  int i, ksiz, snum, start, end, cnum, clip, anum, pnum;
   xmlprintf("<div id=\"estresult\" class=\"estresult\">\n");
-  hits = atoi(cbmapget(hints, "", 0, NULL)) - miss;
+  hits -= miss;
   start = (p_pagenum - 1) * p_perpage;
   end = p_pagenum * p_perpage;
   if(end > dnum) end = dnum;
@@ -758,6 +872,7 @@ static void showresult(ESTDOC **docs, int dnum, CBMAP *hints, ESTCOND *cond, int
   xmlprintf("Results of <strong>%d</strong> - <strong>%d</strong>",
             start + (hits > 0 ? 1 : 0), end);
   xmlprintf(" of about <strong>%d</strong>", hits);
+  if(est_cond_auxiliary_word(cond, "")) xmlprintf(" or more");
   if(p_phrase[0] != '\0' && strlen(p_phrase) < 128)
     xmlprintf(" for <strong>%@</strong>", p_phrase);
   if(g_etime > 0.0) xmlprintf(" (%.3f sec.)", g_etime / 1000.0);
@@ -770,7 +885,19 @@ static void showresult(ESTDOC **docs, int dnum, CBMAP *hints, ESTCOND *cond, int
     while((kbuf = cbmapiternext(hints, &ksiz)) != NULL){
       if(ksiz < 1) continue;
       if(i++ > 0) xmlprintf(", ");
-      xmlprintf("<span class=\"hword\">%s (%s)</span>", kbuf, cbmapget(hints, kbuf, ksiz, NULL));
+      xmlprintf("<span class=\"hword\">%@ (%@%@)</span>", kbuf, cbmapget(hints, kbuf, ksiz, NULL),
+                est_cond_auxiliary_word(cond, kbuf) ? "+" : "");
+    }
+    xmlprintf("</div>\n");
+  }
+  if(scores && scnum > 0){
+    xmlprintf("<div class=\"relkeys\">Related terms: ");
+    for(i = 0; i < scnum && i < g_relkeynum; i++){
+      if(i > 0) xmlprintf(", ");
+      xmlprintf("<a href=\"%@?phrase=%?&amp;attr=%?&amp;attrval=%?&amp;order=%?&amp;perpage=%d"
+                "&amp;clip=%d&amp;qxpnd=%d&amp;prec=%d\" class=\"rword\">%@</a>",
+                g_scriptname, scores[i].word, p_attr, p_attrval, p_order,
+                p_perpage, p_clip, p_qxpnd, p_prec, scores[i].word);
     }
     xmlprintf("</div>\n");
   }
@@ -800,14 +927,15 @@ static void showresult(ESTDOC **docs, int dnum, CBMAP *hints, ESTCOND *cond, int
   xmlprintf("<div class=\"paging\">\n");
   if(clip > 0)
     xmlprintf("<a href=\"%@?phrase=%?&amp;attr=%?&amp;attrval=%?&amp;order=%?&amp;perpage=%d"
-              "&amp;clip=%d&amp;cinc=1&amp;pagenum=%d&amp;similar=%d\" class=\"navi\">"
-              "Include %d Clipped</a>\n", g_scriptname, myphrase, p_attr, p_attrval, p_order,
-              p_perpage, p_clip, p_pagenum, p_similar, clip);
+              "&amp;clip=%d&amp;qxpnd=%d&amp;cinc=-1&amp;prec=%d&amp;pagenum=%d&amp;similar=%d\""
+              " class=\"navi\">Include %d Clipped</a>\n",
+              g_scriptname, myphrase, p_attr, p_attrval, p_order,
+              p_perpage, p_clip, p_qxpnd, p_prec, p_pagenum, p_similar, clip);
   if(p_pagenum > 1){
     xmlprintf("<a href=\"%@?phrase=%?&amp;attr=%?&amp;attrval=%?&amp;order=%?&amp;perpage=%d"
-              "&amp;clip=%d&amp;pagenum=%d&amp;similar=%d\" class=\"navi\">PREV</a>\n",
-              g_scriptname, myphrase, p_attr, p_attrval, p_order,
-              p_perpage, p_clip, p_pagenum - 1, p_similar);
+              "&amp;clip=%d&amp;qxpnd=%d&amp;prec=%d&amp;pagenum=%d&amp;similar=%d\""
+              " class=\"navi\">PREV</a>\n", g_scriptname, myphrase, p_attr, p_attrval, p_order,
+              p_perpage, p_clip, p_qxpnd, p_prec, p_pagenum - 1, p_similar);
   } else {
     xmlprintf("<span class=\"void\">PREV</span>\n");
   }
@@ -819,19 +947,25 @@ static void showresult(ESTDOC **docs, int dnum, CBMAP *hints, ESTCOND *cond, int
         printf("<span class=\"pnow\">%d</span>\n", i);
       } else {
         xmlprintf("<a href=\"%@?phrase=%?&amp;attr=%?&amp;attrval=%?&amp;order=%?&amp;perpage=%d"
-                  "&amp;clip=%d&amp;pagenum=%d&amp;similar=%d\" class=\"pnum\">%d</a>\n",
-                  g_scriptname, myphrase, p_attr, p_attrval, p_order, p_perpage, p_clip,
-                  i, p_similar, i);
+                  "&amp;clip=%d&amp;qxpnd=%d&amp;prec=%d&amp;pagenum=%d&amp;similar=%d\""
+                  " class=\"pnum\">%d</a>\n", g_scriptname, myphrase, p_attr, p_attrval,
+                  p_order, p_perpage, p_clip, p_qxpnd, p_prec, i, p_similar, i);
       }
     }
   }
   if(snum < dnum){
     xmlprintf("<a href=\"%@?phrase=%?&amp;attr=%?&amp;attrval=%?&amp;order=%?&amp;perpage=%d"
-              "&amp;clip=%d&amp;pagenum=%d&amp;similar=%d\" class=\"navi\">NEXT</a>\n",
-              g_scriptname, myphrase, p_attr, p_attrval, p_order,
-              p_perpage, p_clip, p_pagenum + 1, p_similar);
+              "&amp;clip=%d&amp;qxpnd=%d&amp;prec=%d&amp;pagenum=%d&amp;similar=%d\""
+              " class=\"navi\">NEXT</a>\n", g_scriptname, myphrase, p_attr, p_attrval, p_order,
+              p_perpage, p_clip, p_qxpnd, p_prec, p_pagenum + 1, p_similar);
   } else {
     xmlprintf("<span class=\"void\">NEXT</span>\n");
+    if(est_cond_auxiliary_word(cond, ""))
+      xmlprintf("<a href=\"%@?phrase=%?&amp;attr=%?&amp;attrval=%?&amp;order=%?&amp;perpage=%d"
+                "&amp;prec=1&amp;clip=%d&amp;qxpnd=%d&amp;similar=%d\" class=\"navi\">"
+                "Search More Precisely</a>\n",
+                g_scriptname, myphrase, p_attr, p_attrval, p_order,
+                p_perpage, p_clip, p_qxpnd, p_similar);
   }
   xmlprintf("</div>\n");
   xmlprintf("</div>\n");
@@ -941,26 +1075,33 @@ static void showdoc(ESTDOC *doc, const CBLIST *words, CBMAP *cnames, int detail,
   xmlprintf("<dd class=\"doc_navi\">\n");
   xmlprintf("<span class=\"doc_link\">%@</span>\n", turi);
   if(g_candetail)
-    xmlprintf("- <a href=\"%@?phrase=%?&amp;detail=%d&amp;perpage=%d&amp;clip=%d\""
-              " class=\"detail\">[detail]</a>\n",
-              g_scriptname, p_similar > 0 ? "" : p_phrase, id, p_perpage, p_clip);
+    xmlprintf("- <a href=\"%@?phrase=%?&amp;detail=%d&amp;perpage=%d&amp;clip=%d"
+              "&amp;qxpnd=%d&amp;prec=%d\" class=\"detail\">[detail]</a>\n", g_scriptname,
+              p_similar > 0 ? "" : p_phrase, id, p_perpage, p_clip, p_qxpnd, p_prec);
   if(g_smlrvnum > 0)
-    xmlprintf("- <a href=\"%@?similar=%d&amp;perpage=%d&amp;clip=%d\" class=\"similar\">"
-              "[similar]</a>\n", g_scriptname, id, p_perpage, p_clip);
+    xmlprintf("- <a href=\"%@?similar=%d&amp;perpage=%d&amp;clip=%d&amp;qxpnd=%d&amp;prec=%d\""
+              " class=\"similar\">[similar]</a>\n",
+              g_scriptname, id, p_perpage, p_clip, p_qxpnd, p_prec);
   xmlprintf("</dd>\n");
   xmlprintf("</dl>\n");
   if(!detail && shadows && snum > 0){
     for(i = 0; i < snum; i += 2){
-      if(!p_cinc && i >= g_clipview * 2){
+      if(p_cinc >= 0 && p_cinc != id && i >= g_clipview * 2){
         xmlprintf("<div class=\"doc_clip\">\n");
-        xmlprintf("<p>%d more documents clipped...</p>\n", (snum - i) / 2);
+        xmlprintf("<p>%d more documents clipped ... ", (snum - i) / 2);
+        xmlprintf("<a href=\"%@?phrase=%?&amp;attr=%?&amp;attrval=%?&amp;order=%?&amp;perpage=%d"
+                  "&amp;clip=%d&amp;qxpnd=%d&amp;cinc=%d&amp;prec=%d&amp;pagenum=%d"
+                  "&amp;similar=%d#doc_%d\" class=\"include\">[include]</a>",
+                  g_scriptname, p_similar > 0 ? "" : p_phrase, p_attr, p_attrval, p_order,
+                  p_perpage, p_clip, p_qxpnd, id, p_prec, p_pagenum, p_similar, id);
+        xmlprintf("</p>\n");
         xmlprintf("</div>\n");
         *clipp += (snum - i) / 2;
         break;
       }
       if(!(tdoc = est_db_get_doc(g_db, shadows[i], 0))) continue;
       if(g_showscore){
-        sprintf(numbuf, "%1.2f", shadows[i+1] / 10000.0);
+        sprintf(numbuf, "%1.3f", shadows[i+1] >= 9999 ? 1.0 : shadows[i+1] / 10000.0);
         est_doc_add_attr(tdoc, DATTRSCORE, numbuf);
       }
       xmlprintf("<div class=\"doc_clip\">\n");
@@ -998,8 +1139,8 @@ static char *makeshownuri(const char *uri){
 /* show the top */
 static void showinfo(void){
   xmlprintf("<div id=\"estinfo\" class=\"estinfo\">");
-  xmlprintf("Powered by Hyper Estraier %@, with %d documents and %d words.",
-            est_version, est_db_doc_num(g_db), est_db_word_num(g_db));
+  xmlprintf("Powered by <a href=\"%@\">Hyper Estraier</a> %@, with %d documents and %d words.",
+            _EST_PROJURL, est_version, est_db_doc_num(g_db), est_db_word_num(g_db));
   xmlprintf("</div>\n");
 }
 
