@@ -1,29 +1,16 @@
 /*************************************************************************************************
  * Java binding of Hyper Estraier
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
- *                                                                           All rights reserved.
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
- * Redistribution and use in source and binary forms, with or without modification, are
- * permitted provided that the following conditions are met:
- *
- *   * Redistributions of source code must retain the above copyright notice, this list of
- *     conditions and the following disclaimer.
- *   * Redistributions in binary form must reproduce the above copyright notice, this list of
- *     conditions and the following disclaimer in the documentation and/or other materials
- *     provided with the distribution.
- *   * Neither the name of Mikio Hirabayashi nor the names of its contributors may be used to
- *     endorse or promote products derived from this software without specific prior written
- *     permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
- * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
- * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
- * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
- * OF THE POSSIBILITY OF SUCH DAMAGE.
+ * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
+ * the GNU Lesser General Public License as published by the Free Software Foundation; either
+ * version 2.1 of the License or any later version.  Hyper Estraier is distributed in the hope
+ * that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+ * License for more details.
+ * You should have received a copy of the GNU Lesser General Public License along with Hyper
+ * Estraier; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307 USA.
  *************************************************************************************************/
 
 
@@ -43,7 +30,7 @@ public class Cmd {
   // class fields
   //----------------------------------------------------------------
   /* class constants */
-  private static final String PROTVER = "0.9";
+  private static final String PROTVER = "1.0";
   private static final int SEARCHMAX = 10;
   private static final int SNIPWWIDTH = 480;
   private static final int SNIPHWIDTH = 96;
@@ -61,7 +48,8 @@ public class Cmd {
   /**
    * Main routine of the command.
    * @param args command line arguments.  The first argument specifies the name of the sub
-   * command.  "put", "out", "edit", "get", "uriid", "inform", "optimize", "search" are provided.
+   * command.  "put", "out", "edit", "get", "uriid", "inform", "optimize", "merge", "search" are
+   * provided.
    */
   public static void main(String[] args){
     int rv = 0;
@@ -80,6 +68,8 @@ public class Cmd {
       rv = runinform(args);
     } else if(args[0].equals("optimize")){
       rv = runoptimize(args);
+    } else if(args[0].equals("merge")){
+      rv = runmerge(args);
     } else if(args[0].equals("search")){
       rv = runsearch(args);
     } else {
@@ -98,15 +88,16 @@ public class Cmd {
     stderr.println("  java " + Cmd.class.getName() + " arguments...");
     stderr.println("");
     stderr.println("arguments:");
-    stderr.println("  put [-cl] db [file]");
+    stderr.println("  put [-cl] [-ws] db [file]");
     stderr.println("  out [-cl] db expr");
     stderr.println("  edit db expr name [value]");
     stderr.println("  get db expr [attr]");
     stderr.println("  uriid db uri");
     stderr.println("  inform db");
     stderr.println("  optimize [-onp] [-ond] db");
+    stderr.println("  merge [-cl] db target");
     stderr.println("  search [-vu|-va|-vf|-vs] [-gs|-gf|-ga] [-cd] [-ni] [-sf]" +
-                   " [-attr expr] [-ord expr] [-max num] db [phrase]");
+                   " [-attr expr] [-ord expr] [-max num] [-sk num] db [phrase]");
     stderr.println("");
     System.exit(1);
   }
@@ -125,6 +116,8 @@ public class Cmd {
       if(dbname == null && arg.startsWith("-")){
         if(arg.equals("-cl")){
           opts |= Database.PDCLEAN;
+        } else if(arg.equals("-ws")){
+          opts |= Database.PDWEIGHT;
         } else {
           usage();
         }
@@ -268,6 +261,30 @@ public class Cmd {
     if(dbname == null) usage();
     return procoptimize(dbname, opts);
   }
+  /* parse arguments of the merge command */
+  private static int runmerge(String[] args){
+    String dbname = null;
+    String tgname = null;
+    int opts = 0;
+    for(int i = 1; i < args.length; i++){
+      String arg = args[i];
+      if(dbname == null && arg.startsWith("-")){
+        if(arg.equals("-cl")){
+          opts |= Database.MGCLEAN;
+        } else {
+          usage();
+        }
+      } else if(dbname == null){
+        dbname = arg;
+      } else if(tgname == null){
+        tgname = arg;
+      } else {
+        usage();
+      }
+    }
+    if(dbname == null || tgname == null) usage();
+    return procmerge(dbname, tgname, opts);
+  }
   /* parse arguments of the search command */
   private static int runsearch(String[] args){
     String dbname = null;
@@ -275,6 +292,7 @@ public class Cmd {
     List attrs = new ArrayList(3);
     String ord = null;
     int max = SEARCHMAX;
+    int skip = 0;
     int opts = 0;
     boolean cd = false;
     int view = VM_ID;
@@ -310,6 +328,9 @@ public class Cmd {
         } else if(arg.equals("-max")){
           if(++i >= args.length) usage();
           max = Integer.parseInt(args[i]);
+        } else if(arg.equals("-sk")){
+          if(++i >= args.length) usage();
+          skip = Integer.parseInt(args[i]);
         } else {
           usage();
         }
@@ -322,7 +343,7 @@ public class Cmd {
       }
     }
     if(dbname == null) usage();
-    return procsearch(dbname, phrase, attrs, ord, max, opts, cd, view);
+    return procsearch(dbname, phrase, attrs, ord, max, skip, opts, cd, view);
   }
   /* perform the put command */
   private static int procput(String dbname, String file, int opts){
@@ -361,6 +382,7 @@ public class Cmd {
       printerror(dbname + ": " + db.err_msg(db.error()));
       return 1;
     }
+    db.set_informer(new DatabaseInformerImpl());
     if(!db.put_doc(doc, opts)){
       printerror(dbname + ": " + db.err_msg(db.error()));
       db.close();
@@ -379,6 +401,7 @@ public class Cmd {
       printerror(dbname + ": " + db.err_msg(db.error()));
       return 1;
     }
+    db.set_informer(new DatabaseInformerImpl());
     int id = -1;
     try {
       id = Integer.parseInt(expr);
@@ -408,6 +431,7 @@ public class Cmd {
       printerror(dbname + ": " + db.err_msg(db.error()));
       return 1;
     }
+    db.set_informer(new DatabaseInformerImpl());
     int id = -1;
     try {
       id = Integer.parseInt(expr);
@@ -514,13 +538,14 @@ public class Cmd {
     }
     return 0;
   }
-  /* perform the inform command */
+  /* perform the optimize command */
   private static int procoptimize(String dbname, int opts){
     Database db = new Database();
     if(!db.open(dbname, Database.DBWRITER)){
       printerror(dbname + ": " + db.err_msg(db.error()));
       return 1;
     }
+    db.set_informer(new DatabaseInformerImpl());
     if(!db.optimize(opts)){
       printerror(dbname + ": " + db.err_msg(db.error()));
       db.close();
@@ -532,9 +557,28 @@ public class Cmd {
     }
     return 0;
   }
+  /* perform the merge command */
+  private static int procmerge(String dbname, String tgname, int opts){
+    Database db = new Database();
+    if(!db.open(dbname, Database.DBWRITER)){
+      printerror(dbname + ": " + db.err_msg(db.error()));
+      return 1;
+    }
+    db.set_informer(new DatabaseInformerImpl());
+    if(!db.merge(tgname, opts)){
+      printerror(dbname + ": " + db.err_msg(db.error()));
+      db.close();
+      return 1;
+    }
+    if(!db.close()){
+      printerror(dbname + ": " + db.err_msg(db.error()));
+      return 1;
+    }
+    return 0;
+  }
   /* perform the search command */
   private static int procsearch(String dbname, String phrase, List attrs, String ord,
-                                int max, int opts, boolean cd, int view){
+                                int max, int skip, int opts, boolean cd, int view){
     Database db = new Database();
     if(!db.open(dbname, Database.DBREADER)){
       printerror(dbname + ": " + db.err_msg(db.error()));
@@ -548,6 +592,7 @@ public class Cmd {
     }
     if(ord != null) cond.set_order(ord);
     if(max >= 0) cond.set_max(max);
+    if(skip >= 0) cond.set_skip(skip);
     cond.set_options(opts);
     long stime = new GregorianCalendar().getTimeInMillis();
     Result res = db.search(cond);
@@ -635,4 +680,16 @@ public class Cmd {
 
 
 
+/**
+ * Command line utility for the core API.
+ */
+class DatabaseInformerImpl implements DatabaseInformer {
+  /* inform of a database event */
+  public void inform(String message){
+    System.out.println(Cmd.class.getName() + ": INFO: " + message);
+  }
+}
+
+
+
 /* END OF FILE */
