@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Pure Java interface of Hyper Estraier
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  *                                                                           All rights reserved.
  * This file is part of Hyper Estraier.
  * Redistribution and use in source and binary forms, with or without modification, are
@@ -58,8 +58,8 @@ public class Call {
   /**
    * Main routine of the command.
    * @param args command line arguments.  The first argument specifies the name of the sub
-   * command.  "put", "out", "edit", "get", "etch", "uriid", "inform", "search", "setuser", and
-   * "setlink" are provided.
+   * command.  "put", "out", "edit", "get", "etch", "uriid", "inform", "sync", "optimize",
+   "search", "setuser", and "setlink" are provided.
    */
   public static void main(String[] args){
     int rv = 0;
@@ -78,6 +78,10 @@ public class Call {
       rv = runuriid(args);
     } else if(args[0].equals("inform")){
       rv = runinform(args);
+    } else if(args[0].equals("sync")){
+      rv = runsync(args);
+    } else if(args[0].equals("optimize")){
+      rv = runoptimize(args);
     } else if(args[0].equals("search")){
       rv = runsearch(args);
     } else if(args[0].equals("setuser")){
@@ -114,8 +118,13 @@ public class Call {
                    " nurl uri");
     stderr.println("  inform [-proxy host:port] [-tout num] [-auth user pass]" +
                    " nurl");
+    stderr.println("  sync [-proxy host:port] [-tout num] [-auth user pass]" +
+                   " nurl");
+    stderr.println("  optimize [-proxy host:port] [-tout num] [-auth user pass]" +
+                   " nurl");
     stderr.println("  search [-proxy host:port] [-tout num] [-auth user pass]" +
-                   " [-kw] [-sf] [-attr expr] [-ord expr] [-max num] [-dpt num] nurl [phrase]");
+                   " [-sf] [-attr expr] [-ord expr] [-max num] [-sk num]" +
+                   " [-dpt num] [-mask num] nurl [phrase]");
     stderr.println("  setuser [-proxy host:port] [-tout num] [-auth user pass]" +
                    " nurl name mode");
     stderr.println("  setlink [-proxy host:port] [-tout num] [-auth user pass]" +
@@ -372,15 +381,9 @@ public class Call {
     if(nurl == null) usage();
     return procinform(nurl);
   }
-  /* parse arguments of the search command */
-  private static int runsearch(String[] args){
+  /* parse arguments of the sync command */
+  private static int runsync(String[] args){
     String nurl = null;
-    String phrase = null;
-    List attrs = new ArrayList(3);
-    String ord = null;
-    int max = SEARCHMAX;
-    int opts = 0;
-    int depth = 0;
     for(int i = 1; i < args.length; i++){
       String arg = args[i];
       if(nurl == null && arg.startsWith("-")){
@@ -397,8 +400,76 @@ public class Call {
           authname = args[i];
           if(++i >= args.length) usage();
           authpass = args[i];
-        } else if(arg.equals("-kw")){
-          opts |= Condition.ETCH;
+        } else {
+          usage();
+        }
+      } else if(nurl == null){
+        nurl = arg;
+      } else {
+        usage();
+      }
+    }
+    if(nurl == null) usage();
+    return procsync(nurl);
+  }
+  /* parse arguments of the optimize command */
+  private static int runoptimize(String[] args){
+    String nurl = null;
+    for(int i = 1; i < args.length; i++){
+      String arg = args[i];
+      if(nurl == null && arg.startsWith("-")){
+        if(arg.equals("-proxy")){
+          if(++i >= args.length) usage();
+          pxhost = args[i];
+          if(++i >= args.length) usage();
+          pxport = Integer.parseInt(args[i]);
+        } else if(arg.equals("-tout")){
+          if(++i >= args.length) usage();
+          timeout = Integer.parseInt(args[i]);
+        } else if(arg.equals("-auth")){
+          if(++i >= args.length) usage();
+          authname = args[i];
+          if(++i >= args.length) usage();
+          authpass = args[i];
+        } else {
+          usage();
+        }
+      } else if(nurl == null){
+        nurl = arg;
+      } else {
+        usage();
+      }
+    }
+    if(nurl == null) usage();
+    return procoptimize(nurl);
+  }
+  /* parse arguments of the search command */
+  private static int runsearch(String[] args){
+    String nurl = null;
+    String phrase = null;
+    List attrs = new ArrayList(3);
+    String ord = null;
+    int max = SEARCHMAX;
+    int skip = 0;
+    int opts = 0;
+    int depth = 0;
+    int mask = 0;
+    for(int i = 1; i < args.length; i++){
+      String arg = args[i];
+      if(nurl == null && arg.startsWith("-")){
+        if(arg.equals("-proxy")){
+          if(++i >= args.length) usage();
+          pxhost = args[i];
+          if(++i >= args.length) usage();
+          pxport = Integer.parseInt(args[i]);
+        } else if(arg.equals("-tout")){
+          if(++i >= args.length) usage();
+          timeout = Integer.parseInt(args[i]);
+        } else if(arg.equals("-auth")){
+          if(++i >= args.length) usage();
+          authname = args[i];
+          if(++i >= args.length) usage();
+          authpass = args[i];
         } else if(arg.equals("-sf")){
           opts |= Condition.SIMPLE;
         } else if(arg.equals("-attr")){
@@ -410,9 +481,15 @@ public class Call {
         } else if(arg.equals("-max")){
           if(++i >= args.length) usage();
           max = Integer.parseInt(args[i]);
+        } else if(arg.equals("-sk")){
+          if(++i >= args.length) usage();
+          skip = Integer.parseInt(args[i]);
         } else if(arg.equals("-dpt")){
           if(++i >= args.length) usage();
           depth = Integer.parseInt(args[i]);
+        } else if(arg.equals("-mask")){
+          if(++i >= args.length) usage();
+          mask = Integer.parseInt(args[i]);
         } else {
           usage();
         }
@@ -425,7 +502,7 @@ public class Call {
       }
     }
     if(nurl == null) usage();
-    return procsearch(nurl, phrase, attrs, ord, max, opts, depth);
+    return procsearch(nurl, phrase, attrs, ord, max, skip, opts, depth, mask);
   }
   /* parse arguments of the setuser command */
   private static int runsetuser(String[] args){
@@ -683,16 +760,44 @@ public class Call {
     int dnum = node.doc_num();
     int wnum = node.word_num();
     double size = node.size();
-    if(name == null || label == null || dnum < 0 || wnum < 0 || size < 0.0){
+    double ratio = node.cache_usage();
+    if(name == null || label == null || dnum < 0 || wnum < 0 || size < 0.0 || ratio < 0.0){
+      printerror("failed: " + node.status());
+      return 1;
+    }
+    stdout.println(name + "\t" + label + "\t" + dnum + "\t" + wnum + "\t" + (long)size +
+                   "\t" + ratio);
+    return 0;
+  }
+  /* perform the sync command */
+  private static int procsync(String nurl){
+    Node node = new Node();
+    node.set_url(nurl);
+    if(pxhost != null) node.set_proxy(pxhost, pxport);
+    if(timeout > 0) node.set_timeout(timeout);
+    if(authname != null) node.set_auth(authname, authpass);
+    if(!node.sync()){
+      printerror("failed: " + node.status());
+      return 1;
+    }
+    return 0;
+  }
+  /* perform the optimize command */
+  private static int procoptimize(String nurl){
+    Node node = new Node();
+    node.set_url(nurl);
+    if(pxhost != null) node.set_proxy(pxhost, pxport);
+    if(timeout > 0) node.set_timeout(timeout);
+    if(authname != null) node.set_auth(authname, authpass);
+    if(!node.optimize()){
       printerror("failed: " + node.status());
       return 1;
     }
-    stdout.println(name + "\t" + label + "\t" + dnum + "\t" + wnum + "\t" + (long)size);
     return 0;
   }
   /* perform the search command */
   private static int procsearch(String nurl, String phrase, List attrs, String ord,
-                                int max, int opts, int depth){
+                                int max, int skip, int opts, int depth, int mask){
     Node node = new Node();
     node.set_url(nurl);
     if(pxhost != null) node.set_proxy(pxhost, pxport);
@@ -705,8 +810,10 @@ public class Call {
       cond.add_attr((String)attrsit.next());
     }
     if(ord != null) cond.set_order(ord);
-    cond.set_max(max);
+    if(max >= 0) cond.set_max(max);
+    if(skip >= 0) cond.set_skip(skip);
     cond.set_options(opts);
+    cond.set_mask(mask);
     NodeResult nres = node.search(cond, depth);
     Calendar cal = new GregorianCalendar();
     String border = "--------[" + cal.getTimeInMillis() + "]--------";
@@ -727,6 +834,15 @@ public class Call {
       if((value = nres.hint("DOCNUM")) != null) stdout.println("DOCNUM\t" + value);
       if((value = nres.hint("WORDNUM")) != null) stdout.println("WORDNUM\t" + value);
       if((value = nres.hint("TIME")) != null) stdout.println("TIME\t" + value);
+      if((value = nres.hint("TIME#i")) != null) stdout.println("TIME#i\t" + value);
+      for(int i = 0;; i++){
+        String key = "TIME#" + i;
+        if((value = nres.hint(key)) != null){
+          stdout.println(key + "\t" + value);
+        } else {
+          break;
+        }
+      }
       for(int i = 0;; i++){
         String key = "LINK#" + i;
         if((value = nres.hint(key)) != null){
@@ -746,8 +862,8 @@ public class Call {
           String name = (String)it.next();
           stdout.println(name + "=" + rdoc.attr(name));
         }
-        String vector = rdoc.vector();
-        if(vector.length() > 0) stdout.println("%VECTOR\t" + vector);
+        String keywords = rdoc.keywords();
+        if(keywords.length() > 0) stdout.println("%VECTOR\t" + keywords);
         stdout.println("");
         stdout.print(rdoc.snippet());
       }
