@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of mastermod
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -19,7 +19,7 @@
 
 /* private function prototypes */
 static void log_close(void);
-static void db_informer(const char *msg);
+static void db_informer(const char *message, void *opaque);
 static int resdoc_compare_by_score(const void *ap, const void *bp);
 static int resdoc_compare_by_str_asc(const void *ap, const void *bp);
 static int resdoc_compare_by_str_desc(const void *ap, const void *bp);
@@ -42,7 +42,7 @@ int log_level = LL_INFO;
 
 
 /* Open the log file. */
-int log_open(const char *rootdir, const char *path, int level){
+int log_open(const char *rootdir, const char *path, int level, int trunc){
   char mypath[URIBUFSIZ];
   assert(rootdir && path);
   log_level = level;
@@ -55,7 +55,7 @@ int log_open(const char *rootdir, const char *path, int level){
   } else {
     sprintf(mypath, "%s%c%s", rootdir, ESTPATHCHR, path);
   }
-  if(!(log_fp = fopen(mypath, "ab"))) return FALSE;
+  if(!(log_fp = fopen(mypath, trunc ? "wb" : "ab"))) return FALSE;
   if(level == LL_CHECK){
     fclose(log_fp);
     log_fp = NULL;
@@ -100,6 +100,41 @@ void log_print(int level, const char *format, ...){
 }
 
 
+/* Rotete the log file. */
+int log_rotate(const char *rootdir, const char *path){
+  FILE *ifp, *ofp;
+  char mypath[URIBUFSIZ], *wp, iobuf[IOBUFSIZ];
+  int err, year, month, day, hour, minute, second, len;
+  assert(rootdir && path);
+  if(!log_fp || fflush(log_fp) == -1) return FALSE;;
+  err = FALSE;
+  wp = mypath;
+  if((ESTPATHCHR == '/' && path[0] == ESTPATHCHR) ||
+     (ESTPATHCHR == '\\' && ((path[0] >= 'A' && path[0] <= 'Z') ||
+                             (path[0] >= 'a' && path[0] <= 'z')) && path[1] == ':' &&
+      path[2] == '\\')){
+    wp += sprintf(wp, "%s", path);
+  } else {
+    wp += sprintf(wp, "%s%c%s", rootdir, ESTPATHCHR, path);
+  }
+  if(!(ifp = fopen(mypath, "rb"))) return FALSE;
+  cbcalendar(-1, 0, &year, &month, &day, &hour, &minute, &second);
+  sprintf(wp, "-%04d%02d%02d%02d%02d%02d", year, month, day, hour, minute, second);
+  if(!(ofp = fopen(mypath, "wb"))){
+    fclose(ifp);
+    return FALSE;
+  }
+  while((len = fread(iobuf, 1, IOBUFSIZ, ifp)) > 0){
+    fwrite(iobuf, 1, len, ofp);
+  }
+  if(fclose(ofp) == -1) err = TRUE;
+  if(fclose(ifp) == -1) err = TRUE;
+  if(fseek(log_fp, 0, SEEK_SET) == -1 || fflush(log_fp) == -1) err = TRUE;
+  if(ftruncate(fileno(log_fp), 0) == -1) err = TRUE;
+  return err ? FALSE : TRUE;;
+}
+
+
 /* Initialize the root directory. */
 int master_init(const char *rootdir){
   DEPOT *depot;
@@ -111,13 +146,16 @@ int master_init(const char *rootdir){
   err = FALSE;
   sprintf(path, "%s%c%s", rootdir, ESTPATHCHR, METAFILE);
   if((depot = dpopen(path, DP_OWRITER | DP_OCREAT | DP_OTRUNC, MINIBNUM))){
-    if(!dpput(depot, MMKMAGIC, -1, MMKMAGVAL, -1, FALSE)) err = TRUE;
+    if(!dpput(depot, MMKMAGIC, -1, MMKMAGVAL, -1, DP_DKEEP)) err = TRUE;
     if(!dpclose(depot)) err = TRUE;
   } else {
     err = TRUE;
   }
   sprintf(path, "%s%c%s", rootdir, ESTPATHCHR, CONFFILE);
   if((ofp = fopen(path, "wb")) != NULL){
+    fprintf(ofp, "# binding address of TCP (0.0.0.0 means every address)\n");
+    fprintf(ofp, "bindaddr: 0.0.0.0\n");
+    fprintf(ofp, "\n");
     fprintf(ofp, "# port number of TCP\n");
     fprintf(ofp, "portnum: 1978\n");
     fprintf(ofp, "\n");
@@ -130,11 +168,14 @@ int master_init(const char *rootdir){
     fprintf(ofp, "# authorization mode (1:none, 2:admin, 3:all)\n");
     fprintf(ofp, "authmode: 2\n");
     fprintf(ofp, "\n");
+    fprintf(ofp, "# maximum length of data to receive (in kilobytes)\n");
+    fprintf(ofp, "recvmax: 1024\n");
+    fprintf(ofp, "\n");
     fprintf(ofp, "# maximum number of connections at the same time\n");
     fprintf(ofp, "maxconn: 30\n");
     fprintf(ofp, "\n");
     fprintf(ofp, "# idle time to start flushing (in seconds)\n");
-    fprintf(ofp, "idleflush: 30\n");
+    fprintf(ofp, "idleflush: 20\n");
     fprintf(ofp, "\n");
     fprintf(ofp, "# idle time to start synchronizing (in seconds)\n");
     fprintf(ofp, "idlesync: 300\n");
@@ -143,7 +184,7 @@ int master_init(const char *rootdir){
     fprintf(ofp, "sessiontimeout: 600\n");
     fprintf(ofp, "\n");
     fprintf(ofp, "# timeout of search (in seconds)\n");
-    fprintf(ofp, "searchtimeout: 8\n");
+    fprintf(ofp, "searchtimeout: 15\n");
     fprintf(ofp, "\n");
     fprintf(ofp, "# maximum number of documents to send\n");
     fprintf(ofp, "searchmax: 1000\n");
@@ -164,11 +205,24 @@ int master_init(const char *rootdir){
     fprintf(ofp, "proxyport:\n");
     fprintf(ofp, "\n");
     fprintf(ofp, "# path of the log file (relative path or absolute path)\n");
-    fprintf(ofp, "logfile: _log\n");
+    fprintf(ofp, "logfile: %s\n", LOGFILE);
     fprintf(ofp, "\n");
     fprintf(ofp, "# logging level (1:debug, 2:information, 3:warning, 4:error, 5:none)\n");
     fprintf(ofp, "loglevel: 2\n");
     fprintf(ofp, "\n");
+    fprintf(ofp, "# command for backup (absolute path of a command)\n");
+    fprintf(ofp, "backupcmd:\n");
+    fprintf(ofp, "\n");
+    fprintf(ofp, "# scale prediction (1:small, 2:medium, 3:large, 4:huge)\n");
+    fprintf(ofp, "scalepred: 2\n");
+    fprintf(ofp, "\n");
+    fprintf(ofp, "# score expression (1:void, 2:char, 3:int, 4:asis)\n");
+    fprintf(ofp, "scoreexpr: 2\n");
+    fprintf(ofp, "\n");
+    fprintf(ofp, "# attribute indexes (attribute name and data type)\n");
+    fprintf(ofp, "attrindex: @mdate{{!}}seq\n");
+    fprintf(ofp, "attrindex: @title{{!}}str\n");
+    fprintf(ofp, "\n");
     fprintf(ofp, "# document root directory (absolute path of a directory to be public)\n");
     fprintf(ofp, "docroot:\n");
     fprintf(ofp, "\n");
@@ -181,7 +235,7 @@ int master_init(const char *rootdir){
     fprintf(ofp, "# whether to deny all nodes except for trusted nodes (0:no, 1:yes)\n");
     fprintf(ofp, "denyuntrusted: 0\n");
     fprintf(ofp, "\n");
-    fprintf(ofp, "# maximum size of the index cache (in mega bytes)\n");
+    fprintf(ofp, "# maximum size of the index cache (in megabytes)\n");
     fprintf(ofp, "cachesize: 64\n");
     fprintf(ofp, "\n");
     fprintf(ofp, "# maximum number of cached records for document attributes\n");
@@ -196,6 +250,12 @@ int master_init(const char *rootdir){
     fprintf(ofp, "# name of the attribute of the special cache\n");
     fprintf(ofp, "specialcache:\n");
     fprintf(ofp, "\n");
+    fprintf(ofp, "# maximum number of expansion of wild cards\n");
+    fprintf(ofp, "wildmax: 256\n");
+    fprintf(ofp, "\n");
+    fprintf(ofp, "# text size limitation of indexing documents (in kilobytes)\n");
+    fprintf(ofp, "limittextsize: 128\n");
+    fprintf(ofp, "\n");
     fprintf(ofp, "# whole width of the snippet of each shown document\n");
     fprintf(ofp, "snipwwidth: 480\n");
     fprintf(ofp, "\n");
@@ -212,7 +272,7 @@ int master_init(const char *rootdir){
     fprintf(ofp, "smlrvnum: 32\n");
     fprintf(ofp, "\n");
     fprintf(ofp, "# number of documents for delay of keyword extraction\n");
-    fprintf(ofp, "extdelay: 8192\n");
+    fprintf(ofp, "extdelay: 4096\n");
     fprintf(ofp, "\n");
     fprintf(ofp, "# e-mail address of the administrator\n");
     fprintf(ofp, "adminemail: magnus@hyperestraier.gov\n");
@@ -228,6 +288,9 @@ int master_init(const char *rootdir){
     fprintf(ofp, "# whether to use simplified search phrase (0:no, 1:yes)\n");
     fprintf(ofp, "uismplphrase: 1\n");
     fprintf(ofp, "\n");
+    fprintf(ofp, "# tuning parameters for similarity search\n");
+    fprintf(ofp, "uismlrtune: 16 1024 4096\n");
+    fprintf(ofp, "\n");
     if(fclose(ofp) == EOF) err = TRUE;
   } else {
     err = TRUE;
@@ -506,7 +569,8 @@ NMGR *nmgr_new(const char *rootdir){
   log_print(LL_INFO, "starting the node manager");
   nmgr = cbmalloc(sizeof(NMGR));
   nmgr->rootdir = cbmemdup(rootdir, -1);
-  nmgr->nodes = cbmapopen();
+  nmgr->nodes = cbmapopenex(MINIBNUM);
+  nmgr->aidxs = cbmapopenex(MINIBNUM);
   return nmgr;
 }
 
@@ -520,6 +584,7 @@ int nmgr_delete(NMGR *nmgr){
   log_print(LL_INFO, "finishing the node manager");
   err = FALSE;
   if(!nmgr_sync(nmgr, FALSE)) err = TRUE;
+  cbmapclose(nmgr->aidxs);
   cbmapiterinit(nmgr->nodes);
   while((kbuf = cbmapiternext(nmgr->nodes, &ksiz)) != NULL){
     vbuf = cbmapget(nmgr->nodes, kbuf, ksiz, NULL);
@@ -555,7 +620,7 @@ int nmgr_load(NMGR *nmgr, int wmode){
   for(i = 0; i < cblistnum(list); i++){
     file = cblistval(list, i, NULL);
     if(!strcmp(file, ESTCDIRSTR) || !strcmp(file, ESTPDIRSTR)) continue;
-    if(!nmgr_put(nmgr, file, wmode)) err = TRUE;
+    if(!nmgr_put(nmgr, file, wmode, 0)) err = TRUE;
   }
   cblistclose(list);
   return err ? FALSE : TRUE;
@@ -613,8 +678,23 @@ int nmgr_sync(NMGR *nmgr, int phis){
 }
 
 
+/* Add an attribute index to a node manager object. */
+void nmgr_add_aidx(NMGR *nmgr, const char *name, const char *type){
+  int tnum;
+  assert(nmgr && name && type);
+  if(!cbstricmp(type, "str")){
+    tnum = ESTIDXATTRSTR;
+  } else if(!cbstricmp(type, "num")){
+    tnum = ESTIDXATTRNUM;
+  } else {
+    tnum = ESTIDXATTRSEQ;
+  }
+  cbmapput(nmgr->aidxs, name, -1, (char *)&tnum, sizeof(int), TRUE);
+}
+
+
 /* Add a node to a node manager object. */
-int nmgr_put(NMGR *nmgr, const char *name, int wmode){
+int nmgr_put(NMGR *nmgr, const char *name, int wmode, int options){
   NODE node;
   ESTMTDB *db;
   CBLIST *list;
@@ -633,11 +713,16 @@ int nmgr_put(NMGR *nmgr, const char *name, int wmode){
   }
   log_print(LL_INFO, "opening a node (%s): %s", wmode ? "WRITER" : "READER", name);
   sprintf(pbuf, "%s%c%s%c%s", nmgr->rootdir, ESTPATHCHR, NODEDIR, ESTPATHCHR, name);
-  if(!(db = est_mtdb_open(pbuf, wmode ? ESTDBWRITER | ESTDBCREAT : ESTDBREADER, &ecode))){
+  if(!(db = est_mtdb_open(pbuf, wmode ? ESTDBWRITER | ESTDBCREAT | options : ESTDBREADER,
+                          &ecode))){
     log_print(LL_ERROR, "DB-ERROR: %s", est_err_msg(ecode));
     return FALSE;
   }
-  est_mtdb_set_informer(db, db_informer);
+  est_mtdb_set_informer(db, db_informer, NULL);
+  cbmapiterinit(nmgr->aidxs);
+  while((cbuf = cbmapiternext(nmgr->aidxs, &csiz)) != NULL){
+    est_mtdb_add_attr_index(db, cbuf, *(int *)cbmapget(nmgr->aidxs, cbuf, csiz, NULL));
+  }
   node.db = db;
   est_mtdb_add_meta(db, NMKNAME, name);
   node.name = cbmemdup(name, -1);
@@ -721,6 +806,39 @@ int nmgr_out(NMGR *nmgr, const char *name){
 }
 
 
+/* Clear registered documents in a node in a node manager object. */
+int nmgr_clear(NMGR *nmgr, const char *name, int options){
+  NODE *node;
+  CBMAP *admins, *users, *links;
+  const char *vbuf;
+  char *label;
+  if(!(vbuf = cbmapget(nmgr->nodes, name, -1, NULL))) return FALSE;
+  node = (NODE *)vbuf;
+  label = cbmemdup(node->label, -1);
+  admins = cbmapdup(node->admins);
+  users = cbmapdup(node->users);
+  links = cbmapdup(node->links);
+  if(!nmgr_out(nmgr, name) || !nmgr_put(nmgr, name, TRUE, options)){
+    cbmapclose(links);
+    cbmapclose(users);
+    cbmapclose(admins);
+    free(label);
+    return FALSE;
+  }
+  if(!(vbuf = cbmapget(nmgr->nodes, name, -1, NULL))) return FALSE;
+  node = (NODE *)vbuf;
+  cbmapclose(node->links);
+  cbmapclose(node->users);
+  cbmapclose(node->admins);
+  free(node->label);
+  node->label = label;
+  node->admins = admins;
+  node->users = users;
+  node->links = links;
+  return TRUE;
+}
+
+
 /* Get a list of names of nodes in a noder manager object. */
 CBLIST *nmgr_names(NMGR *nmgr){
   CBLIST *names;
@@ -1029,10 +1147,11 @@ static void log_close(void){
 
 
 /* Output the log message of a DB event.
-   `msg' specifies the log message of a DB event. */
-static void db_informer(const char *msg){
-  assert(msg);
-  log_print(LL_INFO, "DB-EVENT: %s", msg);
+   `msg' specifies the log message of a DB event.
+   `opaque' is simply ignored. */
+static void db_informer(const char *message, void *opaque){
+  assert(message);
+  log_print(LL_INFO, "DB-EVENT: %s", message);
 }
 
 
