@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Common modules related to estmaster
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -31,6 +31,7 @@
 
 #define NUMBUFSIZ      32                /* size of a buffer for a number */
 #define URIBUFSIZ      8192              /* size of a buffer for an URI */
+#define IOBUFSIZ       8192              /* size of a buffer for I/O */
 #define HOSTBUFSIZ     256               /* size of a buffer for a host name */
 #define ADDRBUFSIZ     48                /* size of a buffer for an address */
 #define NODENAMEMAX    128               /* maximum length of node name  */
@@ -76,6 +77,20 @@ enum {                                   /* enumeration for running modes */
   LL_CHECK = 6                           /* check to open */
 };
 
+enum {                                   /* enumeration for scale prediction */
+  SP_SMALL = 1,                          /* small */
+  SP_MEDIUM = 2,                         /* medium */
+  SP_LARGE = 3,                          /* large */
+  SP_HUGE = 4                            /* huge */
+};
+
+enum {                                   /* enumeration for scoring expression modes */
+  SE_VOID = 1,                           /* void */
+  SE_CHAR = 2,                           /* char */
+  SE_INT = 3,                            /* int */
+  SE_ASIS = 4                            /* asis */
+};
+
 typedef struct {                         /* type of structure for a user object */
   char *name;                            /* unique name */
   char *passwd;                          /* encrypted password */
@@ -107,6 +122,7 @@ typedef struct {                         /* type of structure for a node object
 typedef struct {                         /* type of structure for a node manager object */
   char *rootdir;                         /* path of the root directory */
   CBMAP *nodes;                          /* map of names and entities */
+  CBMAP *aidxs;                          /* map of attribute indexes */
 } NMGR;
 
 typedef struct {                         /* type of structure for a read/write lock object */
@@ -142,14 +158,22 @@ extern int log_level;
    `rootdir' specifies the path of the root directory.
    `path' specifies the path of the log file.
    `level' specifies the leve of logging.
+   `trunc' specifies whether to truncate the log file.
    The return value is true if success, else it is false. */
-int log_open(const char *rootdir, const char *path, int level);
+int log_open(const char *rootdir, const char *path, int level, int trunc);
 
 
 /* Print formatted string into the log file. */
 void log_print(int level, const char *format, ...);
 
 
+/* Rotete the log file.
+   `rootdir' specifies the path of the root directory.
+   `path' specifies the path of the log file.
+   The return value is true if success, else it is false. */
+int log_rotate(const char *rootdir, const char *path);
+
+
 /* Initialize the root directory.
    `rootdir' specifies the path of the root directory.
    The return value is true if success, else it is false. */
@@ -277,12 +301,20 @@ int nmgr_load(NMGR *nmgr, int wmode);
 int nmgr_sync(NMGR *nmgr, int phis);
 
 
+/* Add an attribute index to a node manager object.
+   `nmgr' specifies a node manager object.
+   `name' specifies the name of a target attribute.
+   `type' specifies the expression of a data type. */
+void nmgr_add_aidx(NMGR *nmgr, const char *name, const char *type);
+
+
 /* Add a node to a node manager object.
    `nmgr' specifies a node manager object.
    `name' specifies the unique name of a node.
    `wmode' specifies whether the node is a writer or a reader.
+   `options' specifies options for the database of the node.
    The return value is true if success, else it is false. */
-int nmgr_put(NMGR *nmgr, const char *name, int wmode);
+int nmgr_put(NMGR *nmgr, const char *name, int wmode, int options);
 
 
 /* Remove a node from a node manager object.
@@ -292,6 +324,14 @@ int nmgr_put(NMGR *nmgr, const char *name, int wmode);
 int nmgr_out(NMGR *nmgr, const char *name);
 
 
+/* Clear registered documents in a node in a node manager object.
+   `nmgr' specifies a node manager object.
+   `name' specifies the unique name of a node.
+   `options' specifies options for the database of the node.
+   The return value is true if success, else it is false. */
+int nmgr_clear(NMGR *nmgr, const char *name, int options);
+
+
 /* Get a list of names of nodes in a noder manager object.
    `nmgr' specifies a node manager object.
    The return value is a list object of names of nodes in a node manager object.
