@@ -1,6 +1,6 @@
 /*************************************************************************************************
- * The core API of Hyper Estraier
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ * System-dependent configurations of Hyper Estraier
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -29,12 +29,30 @@
 #undef FALSE
 #define FALSE          0
 
-#if defined(NOZLIB)
+#if defined(_MYNOZLIB)
 #define ESTUSEZLIB     FALSE
 #else
 #define ESTUSEZLIB     TRUE
 #endif
 
+#if defined(_MYLZO)
+#define ESTUSELZO      TRUE
+#else
+#define ESTUSELZO      FALSE
+#endif
+
+#if defined(_MYBZIP)
+#define ESTUSEBZIP     TRUE
+#else
+#define ESTUSEBZIP     FALSE
+#endif
+
+#if defined(_MYMECAB)
+#define ESTUSEMECAB    TRUE
+#else
+#define ESTUSEMECAB    FALSE
+#endif
+
 
 
 /*************************************************************************************************
@@ -72,6 +90,16 @@
 #define _SYS_HPUX_
 #define ESTSYSNAME  "HP-UX"
 
+#elif defined(__osf)
+
+#define _SYS_TRU64_
+#define _QDBM_SYSNAME  "Tru64"
+
+#elif defined(_AIX)
+
+#define _SYS_AIX_
+#define _QDBM_SYSNAME  "AIX"
+
 #elif defined(__APPLE__) && defined(__MACH__)
 
 #define _SYS_MACOSX_
@@ -129,7 +157,18 @@
 
 #include <regex.h>
 #include <iconv.h>
+
+#if ESTUSEZLIB
 #include <zlib.h>
+#endif
+
+#if ESTUSELZO
+#include <lzo/lzo1x.h>
+#endif
+
+#if ESTUSEBZIP
+#include <bzlib.h>
+#endif
 
 #include "md5.h"
 
@@ -192,8 +231,8 @@
 
 #if ! defined(_QDBM_LIBVER)
 #error QDBM is required but not found.
-#elif _QDBM_LIBVER < 1105
-#error This version of QDBM is too old.  Use 1.8.33 or later.
+#elif _QDBM_LIBVER < 1221
+#error This version of QDBM is too old.  Use 1.8.60 or later.
 #endif
 
 #if defined(__va_copy)
