@@ -2,7 +2,7 @@ require "mkmf"
 
 dir_config('estraier')
 
-ENV["PATH"] = ENV["PATH"] + "/usr/local/bin:.:..:../.."
+ENV["PATH"] = ENV["PATH"] + ":/usr/local/bin:.:..:../.."
 estcflags = `estconfig --cflags`.chomp
 estldflags = `estconfig --ldflags`.chomp
 estlibs = `estconfig --libs`.chomp
